/**
 * @licstart The following is the entire license notice for the
 *  JavaScript code in this page.
 *
 *  splash-js - A standalone javascript program to edit lenex files
 *  Copyright (C) 2016 Stefan Ubbink
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as
 *  published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @licend The above is the entire license notice
 * for the JavaScript code in this page.
 */
;(function () {
    // TODO use it "use strict";
    // see http://javascriptissexy.com/javascript-apply-call-and-bind-methods-are-essential-for-javascript-professionals/
    // and http://javascriptissexy.com/understand-javascripts-this-with-clarity-and-master-it/
    //"use strict";
    // TODO use coding conventions (w3schools.com)

    var GENDER_FEMALE = "F", GENDER_MALE = "M", GENDER_MIXED = "X";
    var GENDER_ALL = "A";

    function onZipError(message) {
        alert("ZIP: " + message);
    };

    function zipReaderFunction(reader) {
        // get all entries from the zip
        reader.getEntries(function(entries) {
            var dom = {}, json = {}, parsedJson = {};

            if (entries.length) {
                // get first entry content as text
                entries[0].getData(new zip.TextWriter(), function(text) {
                    // text contains the entry data as a String
                    dom = parseXml(text);
                    json = xml2json(dom, "");
                    parsedJson = JSON.parse(json);
                    Splash.createDataStructure(parsedJson.LENEX,
                        Splash.lastFileSender);

                    // close the zip reader
                    reader.close(function() {
                        // onclose callback
                    });
                }, function(current, total) {
                    // onprogress callback
                });
            }
        });
    };

    // Check for the various File API support.
    if (window.File && window.FileReader && window.FileList && window.Blob) {
      // Great success! All the File APIs are supported.
    } else {
      alert('The File APIs are not fully supported in this browser. Some things might not work as expected.');
    }

    Splash = {
        version: "0.8",
        debugMode: undefined,
        defaultWarnBeforeClose: true,
        warnBeforeClose: false,
        splashHtml: false,

        lenexDataObj: undefined,
        lastFileSender: undefined,
        aLenexData: [],
        debugDumpNode: document.getElementById('debugDump'),
        meetInfoNode: document.getElementById('meetInfo'),
        clubsInfoNode: document.getElementById('clubsInfo'),
        currentClubIndex: undefined,
        currentRelayIndex: undefined,
        sourceData: undefined,
        athleteInfoNode: {},
        relayInfoNode: {},
        meetInfo: {},
        competition: {},
        swimstyles: [], // This is an array of the event id's
        processedEventIds: [], // Used to store orgeventid's and the current
                                // eventid values
        club: {},
        shouldBeArray: ['AGEGROUP', 'ATHLETE', 'CLUB', 'ENTRY', 'EVENT',
            'FEE', 'HEAT', 'JUDGE', 'MEET', 'RANKING', 'RECORD', 'RECORDLIST',
            'RELAY', 'RELAYPOSITION', 'RESULT', 'RESULT', 'SESSION', 'SPLIT',
            'TIMESTANDARD', 'TIMESTANDARDLIST', 'TIMESTANDARDREF'],
        lifesavingStrokes: { "OBSTACLE": "Obstacle Swim",
            "MANIKIN": "Manikin Carry", "RESCUEMEDLEY": "Rescue Medley",
            "MANIKINFINS": "Manikin Carry with Fins",
            "TOWFINS": "Manikin Tow with Fins",
            "SUPERLIFESAVER": "Super Lifesaver", "LINETHROW": "Line Throw",
            "MANIKINRELAY": "Manikin Relay", "OBSTACLERELAY": "Obstacle Relay",
            "MEDLEYRELAY": "Medley Relay",
            "UNKNOWN": "UNKNOWN"
            // TODO Add more names
        },
        regionCodes: { "NED": {
            "1": "Groningen", "2": "Friesland", "3": "Drenthe",
            "4": "Overijssel", "5": "Gelderland", "6": "Utrecht",
            "7": "Amsterdam - 't Gooi", "8": "Noord Holland",
            "9": "Gouwe Rijnstreek", "10": "Haaglanden", "11": "Rotterdam",
            "12": "Zeeland", "13": "Noord Brabant", "14": "Limburg" },
                "SUI": {
            "RSR": "Region Suisse Romande",
            "RZW": "Region Zentralschweiz-West",
            "RZO": "Region Zentralschweiz-Ost", "ROS": "Region Ostschweiz",
            "RSI": "Regione Svizzera Italiana" }
        },
        // The nationCodes is build from the Lenex_Nation.txt with the
        // following command:
        // sed -e 's#^\([A-Z]\{3\}\)=\(.*\)\r$#"\1": "\2",#g' Lenex_Nation.txt
        nationCodes: {
            "AFG": "Afghanistan", "ALB": "Albania", "ALG": "Algeria",
            "ASA": "American Samoa", "AND": "Andorra", "ANG": "Angola",
            "ANT": "Antigua & Barbuda", "ARG": "Argentina", "ARM": "Armenia",
            "ARU": "Aruba", "AUS": "Australia", "AUT": "Austria",
            "AZE": "Azerbaijan", "BAH": "Bahamas", "BRN": "Bahrain",
            "BAN": "Bangladesh", "BAR": "Barbados", "BLR": "Belarus",
            "BEL": "Belgium", "BIZ": "Belize", "BEN": "Benin",
            "BER": "Bermuda", "BHU": "Bhutan", "BOL": "Bolivia",
            "BIH": "Bosnia-Herzegovina", "BOT": "Botswana", "BRA": "Brazil",
            "IVB": "British Virgin Is.", "BRU": "Brunei", "BUL": "Bulgaria",
            "BUR": "Burkina Faso", "BDI": "Burundi", "CAM": "Cambodia",
            "CMR": "Cameroon", "CAN": "Canada", "CPV": "Cape Verde",
            "CAY": "Cayman Islands", "CAF": "Central African Rep.",
            "CHA": "Chad", "CHI": "Chile", "CHN": "China",
            "TPE": "Chinese Taipei", "COL": "Colombia", "COM": "Comoros",
            "CGO": "Congo", "COK": "Cook Islands", "CRC": "Costa Rica",
            "CRO": "Croatia", "CUB": "Cuba", "CYP": "Cyprus",
            "CZE": "Czech Republic", "PRK": "DPR Korea",
            "COD": "Dem. Rep. of  Congo", "DEN": "Denmark", "DJI": "Djibouti",
            "DMA": "Dominica", "DOM": "Dominican Republic", "ECU": "Ecuador",
            "EGY": "Egypt", "ESA": "El Salvador", "GEQ": "Equatorial Guinea",
            "ERI": "Eritrea", "EST": "Estonia", "ETH": "Ethiopia",
            "FAR": "Faroe Islands", "FIJ": "Fiji", "FIN": "Finland",
            "MKD": "Fr. Yug. Rep. Macedonia", "FRA": "France", "GAB": "Gabon",
            "GAM": "Gambia", "GEO": "Georgia", "GER": "Germany",
            "GHA": "Ghana", "GIB": "Gibraltar", "GBR": "Great Britain",
            "GRE": "Greece", "GRN": "Grenada", "GUM": "Guam",
            "GUA": "Guatemala", "GUI": "Guinea", "GBS": "Guinea-Bissau",
            "GUY": "Guyana", "HAI": "Haiti", "HON": "Honduras",
            "HKG": "Hong Kong", "HUN": "Hungary", "IRI": "I. R.  Iran",
            "ISL": "Iceland", "IND": "India", "INA": "Indonesia",
            "IRQ": "Iraq", "IRL": "Ireland", "ISR": "Israel", "ITA": "Italy",
            "CIV": "Ivory Coast", "JAM": "Jamaica", "JPN": "Japan",
            "JOR": "Jordan", "KAZ": "Kazakhstan", "KEN": "Kenya",
            "KIR": "Kiribati", "KOR": "Korea", "KUW": "Kuwait",
            "KGZ": "Kyrgyzstan", "LAO": "Laos PDR", "LAT": "Latvia",
            "LIB": "Lebanon", "LES": "Lesotho", "LBR": "Liberia",
            "LBA": "Libya", "LIE": "Liechtenstein", "LTU": "Lithuania",
            "LUX": "Luxembourg", "MAC": "Macau", "MAD": "Madagascar",
            "MAW": "Malawi", "MAS": "Malaysia", "MDV": "Maldives",
            "MLI": "Mali", "MLT": "Malta", "MIL": "Marshall Islands",
            "MTN": "Mauritania", "MRI": "Mauritius", "MEX": "Mexico",
            "FSM": "Micronesia", "MDA": "Moldova", "MON": "Monaco",
            "MNE": "Montenegro", "MGL": "Mongolia", "MAR": "Morocco",
            "MOZ": "Mozambique", "MYA": "Myanmar", "NAM": "Namibia",
            "NRU": "Nauru", "NEP": "Nepal", "NED": "The Netherlands",
            "AHO": "Netherlands Antilles", "NZL": "New Zealand",
            "NCA": "Nicaragua", "NIG": "Niger", "NGR": "Nigeria",
            "NMA": "Northern Mariana Islands", "NOR": "Norway", "OMA": "Oman",
            "PAK": "Pakistan", "PLW": "Palau", "PLE": "Palestine",
            "PAN": "Panama", "PNG": "Papua New Guinea", "PAR": "Paraguay",
            "PER": "Peru", "PHI": "Philippines", "POL": "Poland",
            "POR": "Portugal", "PUR": "Puerto Rico", "QAT": "Qatar",
            "ROU": "Romania", "RUS": "Russia", "RWA": "Rwanda",
            "STP": "S. Tome & Principe", "LCA": "Saint Lucia", "SAM": "Samoa",
            "SMR": "San Marino", "KSA": "Saudi Arabia", "SEN": "Senegal",
            "SRB": "Serbia", "SEY": "Seychelles", "SLE": "Sierra Leone",
            "SIN": "Singapore", "SVK": "Slovakia", "SLO": "Slovenia",
            "SOL": "Solomon Islands", "SOM": "Somalia", "RSA": "South Africa",
            "ESP": "Spain", "SRI": "Sri Lanka", "SKN": "St. Kitts & Nevis",
            "VIN": "St. Vincent & Grenadines", "SUD": "Sudan",
            "SUR": "Suriname", "SWZ": "Swaziland", "SWE": "Sweden",
            "SUI": "Switzerland", "SYR": "Syria", "TAH": "Tahiti",
            "TJK": "Tajikistan", "TAN": "Tanzania", "THA": "Thailand",
            "TLS": "Timor-Leste", "TOG": "Togo", "TGA": "Tonga",
            "TRI": "Trinidad &Tobago", "TUN": "Tunisia", "TUR": "Turkey",
            "TKM": "Turkmenistan", "UGA": "Uganda", "UKR": "Ukraine",
            "UAE": "United Arab Emirates", "USA": "United States",
            "URU": "Uruguay", "UZB": "Uzbekistan", "VAN": "Vanuatu",
            "VEN": "Venezuela", "VIE": "Vietnam", "ISV": "Virgin Islands",
            "YEM": "Yemen", "ZAM": "Zambia", "ZIM": "Zimbabwe",
        },

    init: function() {
        // Use window.location.search to see if we need to enable debugging
        // the substring(1) is to remove the first '?' in the string
        var queryString = window.location.search.substring(1);
        var params = {}, queries = [], temp = [], loop = 0;

        // Test if the init is done from the splash.html
        if (window.location.href.indexOf("splash.html") >= 0) {
            this.splashHtml = true;
        }
        this.constructorData = {
            "@version": "3.0",
            "CONSTRUCTOR": {
                "@name": "LifeSaving.nl splash-js editor",
                "@version": this.version,
                "CONTACT": {
                    //"@name": "none",
                    //"@street": "none",
                    //"@city": "none",
                    //"@zip": "none",
                    "@country": "NL",
                    //"@phone": "0",
                    //"@fax": "0",
                    "@email": "splash-js@lifesaving.nl",
                    "@internet": "http://splash-js.lifesaving.nl"
                }
            } // CONSTRUCTOR
        };

        if (this.splashHtml) {
            zip.workerScriptsPath = "./lib/";
        }

        // Parse the arguments that are given to the page
        queries = queryString.split("&");
        for (loop = 0; loop < queries.length; loop++) {
            temp = queries[loop].split('=');
            params[temp[0]] = temp[1];
        }
        // See if debug=1 has been given as agrument, so we can add some debug
        // displays
        if (params.debug) {
            this.debugMode = true;
            document.getElementById('debugPart').style.display = 'initial';
        }
        if (params.nowarn) {
            this.defaultWarnBeforeClose = false;
        }

        // This will handle the filling of the modal for adding/editing a club
        $('#modalClubInfo').on('show.bs.modal', function (eventData) {
            // link or button that triggered the modal
            var relation = $(eventData.relatedTarget);
            // extract info from the data-* attributes (see
            // api.jquery.com/data/ for more information)
            var clubId = relation.data('clubId');
            var club = {};
            var modal = $(this);
            var index = 0, allKeys = _.allKeys(Splash.nationCodes);

            // XXX Test if this will keep the value clear
            modal.find('#saveClub').attr("data-club-id", undefined);
            relation.data('clubId', undefined);

            // Always add the whole country list to the modal options
            for (index in allKeys) {
                modal.find('#clubNation').append(
                    $("<option></option>")
                        .attr("value", allKeys[index])
                        .text(Splash.nationCodes[allKeys[index]])
                );
            }

            // Only fill the other modal fields when there is a clubId
            if (clubId) {
                club = Splash.club[clubId];

                modal.find('.modal-title').text('Club: ' + clubId);
                modal.find('#clubCode').val(club['@code']);
                modal.find('#clubName').val(club['@name']);
                modal.find('#clubNameEn').val(club['@name.en']);
                modal.find('#clubNation').val(club['@nation']);
                modal.find('#clubTeamNumber').val(club['@number']);
                // TODO The region should also be a select element, based on
                // the country
                // http://stackoverflow.com/questions/27631347/how-to-get-selected-item-in-bootstrap-3-dynamic-dropdown-modal-dialog-from-javas
                modal.find('#clubRegion').val(club['@region']);
                modal.find('#clubShortName').val(club['@shortname']);
                modal.find('#clubShortNameEn').val(club['@shortname.en']);
                modal.find('#clubType').val(club['@type']);
                // Add the clubId to the save button to be able to retreive it
                modal.find('#saveClub').attr("data-club-id", clubId);
            }
        });

        // Save the club information
        $('#saveClub').click(function() {
            // extract info from the data-* attributes (see
            // api.jquery.com/data/ for more information)
            var clubId = $('#saveClub').data("clubId");
            var modal = $(document.getElementById('modalClubInfo'));
            var newClubName = "";

            $('#saveClub').data("clubId", undefined);

            // Check that mandatory fields are filled
            if (!clubId) {
                clubId = modal.find('#clubName').val();
                if (Splash.club[clubId]) {
                    // The clubname already exists, so we don't continue and
                    // don't close the modal, so the user can change the name
                    Splash.markField(modal, '#divClubName', ' has-error');
                    alert("Club " + clubId + " already exists.");
                    return;
                } else if (clubId === "") {
                    Splash.markField(modal, '#divClubName', ' has-error');
                    alert("The club needs a name");
                    return;
                }
                Splash.club[clubId] = {};
                // Create a new club based on the given information
            }
            newClubName = Splash.getModalValue(modal, '#clubName');
            // Save the data to the current club
            Splash.club[clubId]['@name'] = newClubName;
            Splash.club[clubId]['@name.en'] = Splash.getModalValue(modal,
                '#clubNameEn');
            Splash.club[clubId]['@nation'] = Splash.getModalValue(modal,
                '#clubNation');
            Splash.club[clubId]['@number'] = Splash.getModalValue(modal,
                '#clubTeamNumber');
            // TODO The region field should be cleared when the country changes
            Splash.club[clubId]['@region'] = Splash.getModalValue(modal,
                '#clubRegion');
            Splash.club[clubId]['@shortname'] = Splash.getModalValue(modal,
                '#clubShortName');
            Splash.club[clubId]['@shortname.en'] = Splash.getModalValue(modal,
                '#clubShortNameEn');
            Splash.club[clubId]['@type'] = Splash.getModalValue(modal,
                '#clubType');

            if (clubId !== newClubName) {
                // 'Rename' the club
                Splash.club[newClubName] = Splash.club[clubId];
                delete Splash.club[clubId];
            }

            // Remove the club id from the modal, to be able to reuse it
            modal.find('#saveClub').removeAttr("data-club-id");
            //modal.find('#saveClub').attr("data-club-id", undefined);
            $('#modalClubInfo').modal('hide');

            // Display the club the usual way
            Splash.displayClubs();
            Splash.setWarnBeforeClose(true);
        });

        // Clear the modal when it is hidden (closed/saved)
        // (http://jsfiddle.net/5LCSU/)
        $('#modalClubInfo').on('hidden.bs.modal', function (e) {
            $(this)
                .find("input,textarea,select")
                    .val('')
                    .end()
                .find("input[type=checkbox], input[type=radio]")
                    .prop("checked", "")
                    .end();
            // Remove any error references
            Splash.clearField($(this), '#divClubName', ' has-error');
        });

        // This will handle the filling of the athlete modal
        $('#modalAthleteInfo').on('show.bs.modal', function (eventData) {
            // link or button that triggered the modal
            var relation = $(eventData.relatedTarget);
            // extract info from the data-* attributes (see
            // api.jquery.com/data/ for more information)
            var athleteIndex = relation.data('athleteIndex');
            var clubId = relation.data('clubId');
            var athleteObj = Splash.getAthleteData(clubId, athleteIndex);
            var index = 0, allKeys = _.allKeys(Splash.nationCodes);
            var athleteIds = {};

            // Update the modal's content
            var modal = $(this);

            // Always add the whole country list to the modal options
            // TODO See if we can cache this list somewhere, because it takes a
            // lot of time to generate it every time
            // Or don't make it a select item by default, but a text with an
            // edit icon next to it, which will then make it an select item
            // glyphicon glyphicon-edit
            // XXX The time seems to be OK for now
            for (index in allKeys) {
                modal.find('#athleteNation').append(
                    $("<option></option>")
                        .attr("value", allKeys[index])
                        .text(Splash.nationCodes[allKeys[index]])
                );
            }

            // Only display values when they are available
            if (!athleteObj) {
                modal.find('.modal-title').text('Add Athlete')
                    .append("<br /><small>" + clubId + "</small>");
            } else {
                modal.find('.modal-title').text('Athlete: ' +
                    athleteObj.athleteName)
                        .append("<br /><small>" + clubId + "</small>");
                modal.find('#athleteAthleteId').val(athleteObj.athleteId);
                modal.find('#athleteFirstName').val(athleteObj.firstName);
                modal.find('#athleteFirstNameEn').val(athleteObj.firstNameEn);
                modal.find('#athleteNamePrefix').val(athleteObj.namePrefix);
                modal.find('#athleteLastName').val(athleteObj.lastName);
                modal.find('#athleteLastNameEn').val(athleteObj.lastNameEn);
                // First remove all checked from both radio buttons
                modal.find('#athleteGenderFemale').prop("checked", false);
                modal.find('#athleteGenderMale').prop("checked", false);
                if (athleteObj.gender === GENDER_FEMALE) {
                    modal.find('#athleteGenderFemale').prop("checked", true);
                } else if (athleteObj.gender === GENDER_MALE) {
                    modal.find('#athleteGenderMale').prop("checked", true);
                } else {
                    modal.find('#athleteGenderFemale').prop("checked", false);
                    modal.find('#athleteGenderMale').prop("checked", false);
                }
                modal.find('#athleteNation').val(athleteObj.nation);
                modal.find('#athleteBirthdate').val(athleteObj.birthdate);
                modal.find('#athleteLicense').val(athleteObj.license);
                modal.find('#athleteLevel').val(athleteObj.level);
                modal.find('#athletePassport').val(athleteObj.passport);
                modal.find('#athleteSwrid').val(athleteObj.swrid);
                athleteIds.athleteIndex = athleteIndex;
            }
            athleteIds.clubId = clubId;
            modal.find('#saveAthlete').data("athleteIds", athleteIds);
        });

        // Save the athlete
        $('#saveAthlete').click(function() {
            // Save the values to the current athlete
            // See also http://www.bootply.com/N2maILYKIT
            // or http://www.bootply.com/bTURSgzRua
            var athleteIds = $('#saveAthlete').data("athleteIds");
            var clubId = athleteIds.clubId;
            var athleteIndex = athleteIds.athleteIndex;
            var modal = $(document.getElementById('modalAthleteInfo'));
            var athleteObj = {}, dontSave = false;

            athleteObj.athleteIndex = athleteIndex;
            athleteObj.athleteId = Splash.getModalValue(modal,'#athleteAthleteId');
            // TODO validate, but only a year is also OK (=YYYY-01-01).
            // https://paulschreiber.com/blog/2007/03/02/javascript-date-validation/
            // http://stackoverflow.com/questions/276479/javascript-how-to-validate-dates-in-format-mm-dd-yyyy
            athleteObj.birthdate = Splash.getModalValue(modal,'#athleteBirthdate');
            athleteObj.firstName = Splash.getModalValue(modal,'#athleteFirstName');
            athleteObj.firstNameEn = Splash.getModalValue(modal,'#athleteFirstNameEn');
            if (modal.find('#athleteGenderMale').prop("checked")) {
                athleteObj.gender = Splash.getModalValue(modal,
                    '#athleteGenderMale');
            } else if (modal.find('#athleteGenderFemale').prop("checked")) {
                athleteObj.gender = Splash.getModalValue(modal,
                    '#athleteGenderFemale');
            } else {
                athleteObj.gender = undefined;
            }
            athleteObj.lastName = Splash.getModalValue(modal,'#athleteLastName');
            athleteObj.lastNameEn = Splash.getModalValue(modal,'#athleteLastNameEn');
            athleteObj.level = Splash.getModalValue(modal,'#athleteLevel');
            athleteObj.license = Splash.getModalValue(modal,'#athleteLicense');
            athleteObj.namePrefix = Splash.getModalValue(modal,'#athleteNamePrefix');
            // Don't convert this nation, because it is not displayed here
            athleteObj.nation = Splash.getModalValue(modal,'#athleteNation');
            athleteObj.passport = Splash.getModalValue(modal,'#athletePassport');
            //athleteObj.swrid = Splash.getModalValue(modal,'#');

            // Check if mandatory fields are filled
            // Mandatory fields are: firstname, lastname, birthdate, gender
            if (!athleteObj.firstName) {
                Splash.markField(modal, '#divFirstName', ' has-error');
                dontSave = true;
            } else {
                Splash.clearField(modal, '#divFirstName', ' has-error');
            }
            if (!athleteObj.lastName) {
                Splash.markField(modal, '#divLastName', ' has-error');
                dontSave = true;
            } else {
                Splash.clearField(modal, '#divLastName', ' has-error');
            }
            if (!athleteObj.gender) {
                Splash.markField(modal, '#divGender', ' has-error');
                dontSave = true;
            } else {
                Splash.clearField(modal, '#divGender', ' has-error');
            }
            if (!athleteObj.birthdate) {
                Splash.markField(modal, '#divBirthdate', ' has-error');
                dontSave = true;
            } else {
                Splash.clearField(modal, '#divBirthdate', ' has-error');
            }

            // There were errors, so we don't save and keep the modal open
            if (dontSave) {
                alert("Please fill all mandatory fields");
                return;
            }
            // Reset the value of the athlete index
            $('#saveAthlete').data("athleteIds", null);

            // Now save the new or changed athlete
            Splash.saveAthlete(clubId, athleteObj);
            // Display the new or changed athlete
            // TODO See if calling the displayClubs is fast enough
            Splash.displayClubs();

            $('#modalAthleteInfo').modal('hide');
            Splash.setWarnBeforeClose(true);
        });

        $('#modalAthleteInfo').on('hidden.bs.modal', function (e) {
            $(this)
                .find("input,textarea,select")
                    .val('')
                    .end()
                .find("input[type=checkbox], input[type=radio]")
                    .prop("checked", "")
                    .end();
            // Reset the values for the gender
            $(this).find('#athleteGenderFemale').val(GENDER_FEMALE);
            $(this).find('#athleteGenderMale').val(GENDER_MALE);

            // Remove any error references
            Splash.clearField($(this), '#divFirstName', ' has-error');
            Splash.clearField($(this), '#divLastName', ' has-error');
            Splash.clearField($(this), '#divGender', ' has-error');
            Splash.clearField($(this), '#divBirthdate', ' has-error');
        });

        // This will handle the filling of the athlete records modal
        $('#modalAthleteRecord').on('show.bs.modal', function (eventData) {
            // link or button that triggered the modal
            var relation = $(eventData.relatedTarget);
            // extract info from the data-* attributes (see
            // api.jquery.com/data/ for more information)
            var athleteIndex = relation.data('athleteIndex');
            var clubId = relation.data('clubId');
            var athleteObj = Splash.getAthleteData(clubId, athleteIndex);
            var athleteIds = {};
            var modal = $(this);

            // Only display values when they are available
            if (!athleteObj) {
                modal.find('.modal-title').text('Add Athlete')
                    .append("<br /><small>" + clubId + "</small>");
            } else {
                modal.find('.modal-title').text('Records for: ' +
                    athleteObj.athleteName)
                        .append("<br /><small>" + clubId + "</small>")
                        .append("<br /><small>Clear a field to remove the entry</small>");
                // Now display all the items
                Splash.displayEntryTimes(modal, athleteObj.ENTRIES,
                    athleteObj.athleteId);
                athleteIds.athleteIndex = athleteIndex;
            }
            athleteIds.clubId = clubId;
            modal.find('#saveAthleteRecord').data("athleteIds", athleteIds);
        });

        // Save the athlete
        $('#saveAthleteRecord').click(function() {
            // Save the values to the current athlete
            var athleteIds = $('#saveAthleteRecord').data("athleteIds");
            var clubId = athleteIds.clubId;
            var athleteIndex = athleteIds.athleteIndex;
            var numberOfRecords = $('#recordAdd').data("numberOfRecords");
            var modal = $(document.getElementById('modalAthleteRecord'));
            var athleteObj = {}, dontSave = false;
            var recordInfo = {};
            var loop = 0, removedRecords = 0;

            athleteObj.athleteIndex = athleteIndex;
            athleteObj.athleteId = Splash.getModalValue(modal,'#athleteAthleteId');
            // Reset the value of the athlete index
            $('#saveAthleteRecord').data("athleteIds", null);

            // Now save the new or changed records
            for (loop = 0; loop < numberOfRecords; loop++) {
                recordInfo = {};
                recordInfo.index = loop - removedRecords;
                recordInfo.time = Splash.getModalValue(modal,'#recordEntryTime' + loop);
                recordInfo.name = Splash.getModalValue(modal,'#recordName' + loop);
                if (!Splash.saveAthleteRecord(clubId, athleteIndex,
                    recordInfo)) {
                    // Because an entry is removed, we need to decrease the
                    // numberOfRecords var
                    numberOfRecords--;
                    removedRecords++;
                }
            }

            $('#modalAthleteRecord').modal('hide');
            Splash.setWarnBeforeClose(true);
        });

        $('#modalAthleteRecord').on('hidden.bs.modal', function (e) {
            $(this)
                .find("input,textarea,select")
                    .val('')
                    .end()
                .find("input[type=checkbox], input[type=radio]")
                    .prop("checked", "")
                    .end();
        });

        // This will handle the filling of the athlete entry into the
        // competition
        $('#modalAthleteCompetition').on('show.bs.modal', function (eventData) {
            // link or button that triggered the modal
            var relation = $(eventData.relatedTarget);
            // extract info from the data-* attributes (see
            // api.jquery.com/data/ for more information)
            var athleteIndex = relation.data('athleteIndex');
            var clubId = relation.data('clubId');
            var athleteObj = Splash.getAthleteData(clubId, athleteIndex);
            var athleteIds = {};
            var revDate = null, athleteDate = null;
            var modal = $(this);

            // Determine the age of the athlete
            try {
                if (Splash.meetInfo.AGEDATE['@type'] == "YEAR") {
                    // We only need the year, so we use January 1st (01-01)
                    // for the date (=0,1 in JavaScript)
                    revDate = new Date(new Date(
                        Splash.meetInfo.AGEDATE['@value']).getFullYear(), 0, 1);
                } else {
                    revDate = new Date(Splash.meetInfo.AGEDATE['@value']);
                }
                athleteDate = new Date(athleteObj.birthdate);
                athleteObj.age = revDate.getFullYear()
                    - athleteDate.getFullYear();
            } catch (e) {
                alert("No competition is loaded");
                $('#modalAthleteCompetition').modal('hide');
                return;
            }

            // Only display values when they are available
            if (!athleteObj) {
                // XXX This should not be possible
                modal.find('.modal-title').text('Add Athlete')
                    .append("<br /><small>" + clubId + "</small>");
            } else {
                modal.find('.modal-title').text('Entries for: ' +
                    athleteObj.athleteName)
                        .append("<br /><small>" + clubId + "</small>");
                // Now display all the items
                Splash.displayAthleteEvents(modal, athleteObj);

                athleteIds.athleteIndex = athleteIndex;
            }
            athleteIds.clubId = clubId;
            modal.find('#saveAthleteCompetition').data("athleteIds", athleteIds);
        });

        // Save the athlete subscriptions
        $('#saveAthleteCompetition').click(function() {
            // Save the entry for the athlete
            var athleteIds = $('#saveAthleteCompetition').data("athleteIds");
            var clubId = athleteIds.clubId;
            var athleteIndex = athleteIds.athleteIndex;
            var numberOfEvents = $('#saveAthleteCompetition').data("numberOfEvents");
            var modal = $(document.getElementById('modalAthleteCompetition'));
            var athleteObj = {}, dontSave = false;
            var eventInfo = {}, eventData = {};
            var loop = 0;
            var athleteInfo = {}, athleteData = {};

            athleteObj.athleteIndex = athleteIndex;
            athleteObj.athleteId = Splash.getModalValue(modal,'#athleteAthleteId');
            // Reset the value of the athlete index
            $('#saveAthleteCompetition').data("athleteIds", null);

            // Now save the new or changed entries
            for (loop = 0; loop < numberOfEvents; loop++) {
                eventData = modal.find('#athleteEventData' + loop)
                    .data("eventData");
                eventInfo = {};
                try {
                    eventInfo.eventId = eventData.eventId;
                    eventInfo.name = eventData.name;
                    eventInfo.time = Splash.getModalValue(modal,
                        '#athleteEntryTimes' + loop);
                } catch (e) {
                    // This is when the event was not even on the modal,
                    // so we can skip the rest
                    continue;
                }
                if (modal.find('#athleteCompCheckbox' + loop)
                    .prop("checked")) {
                    eventInfo.ageGroupId = eventData.ageGroupId;
                    eventInfo.enteredTime = Splash.getModalValue(modal,
                        '#athleteCompTime' + loop);
                    if (eventInfo.time == "-1") {
                        eventInfo.time = "NT";
                        eventInfo.index = undefined;
                    } else {
                        // It is an index in the current athlete records
                        // Use the right value for an index, because it should
                        // be an index in the athletes' records
                        // It is an string "I-#" where # is the index
                        eventInfo.index = eventInfo.time.slice(2);
                    }
                    if (eventInfo.enteredTime) {
                        // Overwrite the selected time with the entered time
                        eventInfo.time = eventInfo.enteredTime;
                    }
                } else {
                    // The item is unchecked and has to be removed
                    // from the subscribed events
                    eventInfo.remove = true;
                    eventInfo.index = eventInfo.time.slice(2);
                }
                // TODO Find a better way to handle the unchecking of an event
                // If no time is given, it wasn't checked now or before
                if (eventInfo.time != "-1") {
                    Splash.saveAthleteRecord(clubId, athleteIndex, eventInfo);
                    athleteData = Splash.getAthleteData(
                        Splash.currentClubIndex, athleteIndex);

                    athleteInfo.eventId = eventInfo.eventId;
                    athleteInfo.name = athleteData.athleteName;
                    athleteInfo.entryTime = eventInfo.time;
                    athleteInfo.entryTime = "time: TODO"; // This is temporary
                    athleteInfo.id = athleteData.athleteId;
                    athleteInfo.remove = eventInfo.remove;
                    Splash.displayAthleteAtEvent(athleteInfo);
                }
            }

            $('#modalAthleteCompetition').modal('hide');
            Splash.setWarnBeforeClose(true);
        });

        $('#modalAthleteCompetition').on('hidden.bs.modal', function (e) {
            $(this)
                .find("input,textarea,select")
                    .val('')
                    .end()
                .find("input[type=checkbox], input[type=radio]")
                    .prop("checked", "")
                    .end();
        });

        $('#modalRelayInfo').on('show.bs.modal', function (eventData) {
            // link or button that triggered the modal
            var relation = $(eventData.relatedTarget);
            // extract info from the data-* attributes (see
            // api.jquery.com/data/ for more information)
            // TODO Make sure the edit also sends this data
            var eventDataObj = relation.data('eventData');
            //var clubId = relation.data('clubId');
            var modal = $(this);
            var athleteLoop = 0, athleteId = 0;
            var divErrors = document.getElementById("divRelayErrors");
            var nrInput = document.createElement("input");
            var selectInput, swimmers, title = "";
            var selectClub, selectClubID = "selectClub";

            if (!eventDataObj) {
                eventDataObj = {};
                eventDataObj.text = relation.data('text');
                eventDataObj.eventId = relation.data('eventId');
                eventDataObj.nrAthletes = relation.data('nrAthletes');
                eventDataObj.entryTime = relation.data('entryTime');
                eventDataObj.clubId = relation.data('clubId');
                eventDataObj.relayIndex = relation.data('relayIndex');
                eventDataObj.entryIndex = relation.data('entryIndex');
                eventDataObj.teamNumber = relation.data('teamNumber');
                eventDataObj.gender = relation.data('gender');
            }

            title = "Add Relay Team";
            if (eventDataObj.entryTime) {
                title = "Edit Relay Team";
            }
            modal.find('.modal-title').text(title)
                .append("<br /><small>" + eventDataObj.text + "</small>");

            // Loop through all athletes to create a list which can be
            // used in the dropdown list to select the athlete
            selectInput = Splash.getAthletesForRelay(eventDataObj.gender,
                undefined);

            // Create an error message when there is no club loaded
            if (!selectInput) {
                divErrors.textContent = "Unable to add a team, because "
                    + "no club is loaded";
            } else {
                divErrors.textContent = "";
                var inputEntryTime = document.getElementById("relayEntryTime");
                if (eventDataObj.entryTime) {
                    inputEntryTime.value = eventDataObj.entryTime;
                } else {
                    inputEntryTime.value = "";
                }

                var inputTeamNumber = document.getElementById("relayTeamNumber");
                if (eventDataObj.teamNumber) {
                    inputTeamNumber.value = eventDataObj.teamNumber;
                } else {
                    inputTeamNumber.value = "1";
                }

                // Add a select box for the club
                // First try to get the element, if it's not there, create a
                // new one
                selectClub = document.getElementById(selectClubID);
                if (!selectClub) {
                    selectClub = Splash.getClubsForRelay(eventDataObj.clubId);
                    selectClub.id = selectClubID;

                    modal.find('#divRelayEntryTime').append(selectClub);
                } else {
                    // Select the correct club
                    modal.find('#' + selectClubID).val(eventDataObj.clubId);
                }

                swimmers = document.getElementById("divRelaySwimmers");
                // Clear the html
                swimmers.innerHTML = "";

                // Create a loop to create the number of athletes needed for
                // the relay
                for (athleteLoop = 0; athleteLoop < eventDataObj.nrAthletes;
                    athleteLoop++) {
                    var nrInput = document.createElement("input");
                    athleteId = Splash.getRelayAthleteId(eventDataObj,
                        athleteLoop);
                    //var newSelectInput = {};
                    //newSelectInput = selectInput;
                    // TODO We don't want to call this function every time, but
                    // if we don't do this, there will only be 1 select box in
                    // the output
                    var newSelectInput = Splash.getAthletesForRelay(
                            eventDataObj.gender, athleteId);

                    nrInput.type = "number";
                    //nrInput.size = 50; // this does not work
                    nrInput.style.width = "50px"; // = style="width:50px"
                    nrInput.value = athleteLoop + 1;
                    nrInput.id = "relayPosition" + athleteLoop;
                    newSelectInput.id = "relayAthleteId" + athleteLoop;

                    swimmers.appendChild(nrInput);
                    //swimmers.appendChild(selectInput);
                    swimmers.appendChild(newSelectInput);
                    swimmers.appendChild(document.createElement("br"));
                }
            }
            modal.find('#saveRelayTeam').data("eventData", eventDataObj);
        });

        // Save the relay team
        $('#saveRelayTeam').click(function() {
            // Save the entry for the athlete
            var eventData = $('#saveRelayTeam').data("eventData");
            var clubId = eventData.clubId, newClubId = undefined;
            var nrAthletes = eventData.nrAthletes;
            var modal = $(document.getElementById('modalRelayInfo'));
            var loop = 0, relayData = {};
            var position = 0, athleteId = 0;
            var athlete = {}, athleteIndex;

            // Reset the value of the event data
            $('#saveRelayTeam').data("eventData", null);

            relayData.entryTime = Splash.getModalValue(modal,
                '#relayEntryTime');
            relayData.teamNumber = Splash.getModalValue(modal,
                '#relayTeamNumber');
            newClubId = Splash.getModalValue(modal, '#selectClub');
            if (newClubId) {
                clubId = newClubId;
            }
            relayData.clubId = clubId;
            relayData.eventId = eventData.eventId;
            relayData.gender = eventData.gender;
            relayData.relayIndex = eventData.relayIndex;
            relayData.entryIndex = eventData.entryIndex;
            relayData.nrAthletes = nrAthletes;
            relayData.names = "";
            relayData.positions = [];

            for (loop = 0; loop < nrAthletes; loop++) {
                position = Splash.getModalValue(modal, '#relayPosition' + loop);
                athleteId = Splash.getModalValue(modal, '#relayAthleteId' + loop);
                relayData.positions[loop] = {};
                relayData.positions[loop].number = position;
                relayData.positions[loop].athleteId = athleteId;
                // Create the text to display at the meet
                athleteIndex = Splash.getAthleteIndex(athleteId, clubId);
                athlete = Splash.getAthleteData(clubId, athleteIndex);
                if (athlete === undefined) {
                    // TODO handle the selection of a athlete from a different
                    // club
                    alert("Selecting a athlete from a different club"
                        + "is not yet supported");
                    $('#modalRelayInfo').modal('hide');
                    return;
                }
                if (athlete.namePrefix) {
                    relayData.names += athlete.namePrefix + " ";
                }
                relayData.names += athlete.lastName;
                if (loop != nrAthletes - 1) {
                    // Only add a , when other names will be added
                    relayData.names += ", ";
                }
            }

            Splash.saveRelayTeam(clubId, relayData);

            // redisplay the relay teams
            // TODO See if calling the displayClubs is fast enough
            Splash.displayClubs();

            // If there is an event id, it has been added from the meet, so
            // display the team there
            if (eventData.eventId) {
                //relayData.id = "relayId";
                Splash.displayTeamAtEvent(relayData);
            }

            $('#modalRelayInfo').modal('hide');
            Splash.setWarnBeforeClose(true);
        });

        $('#modalRelayInfo').on('hidden.bs.modal', function (e) {
            $(this)
                .find("input,textarea,select")
                    .val('')
                    .end()
                .find("input[type=checkbox], input[type=radio]")
                    .prop("checked", "")
                    .end();
        });

        // This will handle the filling of the athlete entry into the
        // competition
        $('#modalRelayCompetition').on('show.bs.modal', function (eventData) {
            // link or button that triggered the modal
            var relation = $(eventData.relatedTarget);
            // extract info from the data-* attributes (see
            // api.jquery.com/data/ for more information)
            var eventDataObj = relation.data('eventData');
            var modal = $(this);

            if (!eventDataObj) {
                eventDataObj = {};
                eventDataObj.text = relation.data('text');
                eventDataObj.eventId = relation.data('eventId');
                eventDataObj.nrAthletes = relation.data('nrAthletes');
                eventDataObj.entryTime = relation.data('entryTime');
                eventDataObj.clubId = relation.data('clubId');
                eventDataObj.id = relation.data('relayId');
                eventDataObj.relayIndex = relation.data('relayIndex');
                eventDataObj.entryIndex = relation.data('entryIndex');
                eventDataObj.teamNumber = relation.data('teamNumber');
                eventDataObj.gender = relation.data('gender');
            }

            modal.find('.modal-title').text('Entry for: ' +
                eventDataObj.text)
                    .append("<br /><small>" + eventDataObj.clubId + "</small>");

            // TODO Determine the age of the team (see the athlete's
            // competition selection)

            if (!Splash.meetInfo.SESSIONS) {
                alert("No competition is loaded");
                $('#modalRelayCompetition').modal('hide');
                return;
            }

            // TODO Add ENTRIES to the eventDataObj ???
            Splash.displayRelayEvents(modal, eventDataObj);

            modal.find('#saveRelayCompetition').data("eventData", eventDataObj);
        });

        // Save the athlete subscriptions
        $('#saveRelayCompetition').click(function() {
            // Save the entry for the athlete
            var eventData = $('#saveRelayCompetition').data("eventData");
            var clubId = eventData.clubId, newClubId = undefined;
            //var nrAthletes = eventData.nrAthletes;
            var modal = $(document.getElementById('modalRelayCompetition'));
            var loop = 0, relayData = {};
            //var position = 0, athleteId = 0;
            //var athlete = {}, athleteIndex;

            // TODO This is always "undefined"?
            var teamId = Splash.getModalValue(modal, '#teamRelayId');
            // The event id for the selected event
            var relayEvent = Splash.getModalValue(modal, '#relayEvents');
            relayData = eventData;
            relayData.id = teamId;
            relayData.eventId = relayEvent;
            if (!modal.find('#relayCompCheckbox').prop("checked")) {
                // The event should be removed
                relayData.remove = true;
                //relayData.eventId = undefined;
            }
            Splash.saveRelayTeam(clubId, relayData);
            Splash.displayTeamAtEvent(relayData);
            // redisplay the relay teams, to update the information for editing
            Splash.displayClubs();

            $('#modalRelayCompetition').modal('hide');
            Splash.setWarnBeforeClose(true);
        });

        $('#modalRelayCompetition').on('hidden.bs.modal', function (e) {
            $(this)
                .find("input,textarea,select")
                    .val('')
                    .end()
                .find("input[type=checkbox], input[type=radio]")
                    .prop("checked", "")
                    .end();
        });

        // TODO I don't know if this works, from
        // http://stackoverflow.com/questions/16402390/bootstrap-control-with-multiple-data-toggle
        // and see http://stackoverflow.com/questions/18016851/how-to-set-multiple-data-toggle-for-a-link-in-bootstrap
        $('[data-tooltip="tooltip"]').tooltip();
    },

    markField: function(thisArg, className, classValue) {
        thisArg.find(className).attr('class',
            function(index, curVal) {
                return curVal + classValue;
            });
    },

    clearField: function(thisArg, className, classValue) {
        thisArg.find(className).attr('class',
            function(index, curVal) {
                return curVal.replace(classValue,"");
            });
    },

    // Return undefined when the key has no value, to prevent from adding empty
    // values to the xml output.
    getModalValue: function(modal, key) {
        var value = modal.find(key).val();
        if (!value) {
            value = undefined;
        }
        return value;
    },

    getNation: function(codeOrText) {
        var result = codeOrText;
        if (!codeOrText) {
            return result;
        }
        // Test if the code is less than 3 characters and all uppercase
        if (codeOrText.length <= 3 && codeOrText === codeOrText.toUpperCase()) {
            // get the text from the nationCodes
            result = this.nationCodes[codeOrText];
        } else {
            // get the code from the nationCodes
            result = _.invert(this.nationCodes)[codeOrText];
        }
        // If the result is undefined, the code or country was not in the
        // nationCodes, so we return the original text
        if (!result) {
            result = codeOrText;
        }
        return result;
    },

    getRegionText: function(regionNumber, country) {
        var result = regionNumber;

        if (!regionNumber || !country) {
            return result;
        }
        try {
            result = this.regionCodes[country][regionNumber] + " (" +
                regionNumber + ")";
        } catch (e) {
            result = regionNumber;
        }

        return result;
    },

    getTreeData: function(obj, dataString) {
        // TODO test functionparameters and give them a value
        var keyName = "", index = 0, allKeys = _.allKeys(obj);
        //console.log("TD;allKeys: " + allKeys);

        for (index in allKeys) {
            keyName = allKeys[index];
            if (_.isObject(obj[keyName])) {
                dataString += "E: " + keyName + "<br />";

                dataString  = this.getTreeData(obj[keyName], dataString);
            } else {
                dataString += keyName + ": " + obj[keyName] + "<br />";
            }
        }
        return dataString;
    },

    displayTreeData: function() {
        // TODO only display the data we didn't display before.
        this.debugDumpNode.innerHTML = this.getTreeData(this.aLenexData, "");
    },

    storeClub: function(oClub, index, allClubs) {
        // TODO Convert Entry items to own definition, so we can save and load
        // them
        // XXX At the moment storing the swimstyle is enough information
        // <EVENT eventid="1058" daytime="11:15" gender="F" number="1"
        //  order="1" round="TIM" preveventid="-1">
        //   <SWIMSTYLE distance="200" relaycount="1" swimstyleid="805"
        //      name="200m obstacle swim" stroke="UNKNOWN" code="200 obs." />
        // <ENTRY eventid="1058" entrytime="00:02:10.11">
        //   <MEETINFO course="LCM" date="2015-01-31" city="Eindhoven"
        //   nation="NED" />
        // </ENTRY>
        //
        // New:
        // <ENTRY localeventid="SJS###" orgeventid="1058"
        //  entrytime="00:02:10.11"
        //   <MEETINFO course="LCM" date="2015-01-31" city="Eindhoven"
        //      nation="NED" />
        //   <SWIMSTYLE ...>
        // </ENTRY>
        var clubName = oClub['@name'];
        var athlete = undefined, relay = undefined;

        this.club[clubName] = oClub;
        this.currentClubIndex = oClub['@name'];
        // Get the athlete information in the Splash.club array
        if (oClub.ATHLETES) {
            athlete = oClub.ATHLETES.ATHLETE;
            // Loop through the Athletes
            this.handleObjectOrArray(this.getAthleteEntries, athlete,
                this);
        }
        if (oClub.RELAYS) {
            relay = oClub.RELAYS.RELAY;
            // Loop through the relays
            this.handleObjectOrArray(this.getRelayEntries, relay, this);
        }
    },

    getAthleteEntries: function(oAthlete, index, allAthletes) {
        // XXX See getRelayEntries, since it should do the same for relays
        var clubName = this.currentClubIndex;
        var entry = undefined, entryEventId = undefined;
        var entryData = undefined, entries = undefined, loop = 0;
        var clubEntries = undefined;

        try {
            clubEntries = this.club[clubName].ATHLETES.ATHLETE[index].ENTRIES;
        } catch (e) {
            // If there are no entries there is nothing to do
            return;
        }

        this.currentAthleteIndex = index;
        if (!oAthlete.ENTRIES) {
            // There are no entries for this athlete
            return;
        }
        entryData = oAthlete.ENTRIES.ENTRY;
        if (!Array.isArray(entryData)) {
            entryEventId = entryData['@eventid'];
            if (entryEventId) {
                clubEntries.ENTRY['@orgeventid'] = entryEventId;
                // Get the swimstyle from the swimstyles in the meeting data
                clubEntries.ENTRY.SWIMSTYLE = this.swimstyles[entryEventId];

                delete clubEntries.ENTRY['@eventid'];
            }
            entryEventId = undefined;
        } else {
            // Loop through all the entries
            for (loop=0; loop<entryData.length; loop++) {
                entry = entryData[loop];
                entryEventId = entry['@eventid'];

                if (entryEventId) {
                    clubEntries.ENTRY[loop]['@orgeventid'] = entryEventId;
                    // Get the swimstyle from the swimstyles in the meeting
                    // data
                    clubEntries.ENTRY[loop].SWIMSTYLE =
                        this.swimstyles[entryEventId];

                    // Remove the original eventid, so we know this is different
                    delete clubEntries.ENTRY[loop]['@eventid'];
                }
                entryEventId = undefined;
            }
        }
    },

    getRelayEntries: function(oRelay, index, allRelays) {
        // XXX See getAthleteEntries, since it should do the same for athletes
        var clubName = this.currentClubIndex;
        var entry = undefined, entryEventId = undefined;
        var entryData = undefined, entries = [], loop = 0;
        var clubEntries = undefined;

        try {
            clubEntries = this.club[clubName].RELAYS.RELAY[index].ENTRIES;
        } catch (e) {
            // If there are no entries there is nothing to do
            return;
        }
        this.currentRelayIndex = index;
        if (!oRelay.ENTRIES) {
            // There are no entries for this athlete
            return;
        }
        entryData = oRelay.ENTRIES.ENTRY;
        if (!Array.isArray(entryData)) {
            entryEventId = entryData['@eventid'];
            if (entryEventId) {
                clubEntries.ENTRY['@orgeventid'] = entryEventId;
                if (this.swimstyles[entryEventId]) {
                    clubEntries.ENTRY.SWIMSTYLE = this.swimstyles[entryEventId];
                }
                delete clubEntries.ENTRY['@eventid'];
            }
            entryEventId = undefined;
        } else {
            // Loop through all the entries
            for (loop=0; loop<entryData.length; loop++) {
                entry = entryData[loop];
                entryEventId = entry['@eventid'];

                if (entryEventId) {
                    clubEntries.ENTRY[loop]['@orgeventid'] = entryEventId;
                    // Get the swimstyle from the swimstyles in the meeting
                    // data
                    if (this.swimstyles[entryEventId]) {
                        clubEntries.ENTRY[loop].SWIMSTYLE =
                            this.swimstyles[entryEventId];
                    }
                    //clubEntries.ENTRY[0]['@localeventid'] = entry['@eventid'];

                    // Remove the original eventid, so we know this is
                    // different
                    delete clubEntries.ENTRY[loop]['@eventid'];
                }
                entryEventId = undefined;
            }
        }
    },

    displaySingleEntryTime: function(formItem, itemNumber, entryData) {
        var labelEntryTime = document.createElement('label');
        var inputEntryTime = document.createElement('input');
        var divEntryTime = document.createElement('div');
        var labelRecordName = document.createElement('label');
        var inputRecordName = document.createElement('input');
        var divRecordName = document.createElement('div');

        // TODO Maybe add a button to view/edit details of the record (collapse)
        $(labelRecordName).attr({
            'for': 'recordName' + itemNumber,
            'class': 'control-label'
        });
        labelRecordName.textContent = "*Name:";
        $(inputRecordName).attr({
            'id': 'recordName' + itemNumber,
            'type': 'text',
            'class': 'form-control',
            'placeholder': 'Name',
            'value': entryData.entryName
        });
        divRecordName.id = "divRecordName" + itemNumber;
        divRecordName.className = "form-group input-group-sm";
        divRecordName.appendChild(labelRecordName);
        divRecordName.appendChild(inputRecordName);

        $(labelEntryTime).attr({
            'for': 'recordEntryTime' + itemNumber,
            'class': 'control-label'
        });
        labelEntryTime.textContent = "*Time:";
        $(inputEntryTime).attr({
            'id': 'recordEntryTime' + itemNumber,
            'type': 'text',
            'class': 'form-control',
            'placeholder': 'HH:MM:SS.ss or NT',
            'size': '14',
            'value': entryData.entryTime
        });
        divEntryTime.id = "divRecordEntryTime" + itemNumber;
        divEntryTime.className = "form-group input-group-sm";
        divEntryTime.appendChild(labelEntryTime);
        divEntryTime.appendChild(inputEntryTime);

        formItem.appendChild(divRecordName);
        formItem.appendChild(divEntryTime);
        formItem.innerHTML += "<br />";
    },

    addSingleEntryTime: function(formId, recordNumber) {
        // TODO Make sure the previous entered information is not cleared
        var entryData = {};
        var formItem = document.getElementById(formId);
        var recordAddButton = document.getElementById('recordAdd');

        entryData.entryTime = "";
        entryData.entryName = "";
        this.displaySingleEntryTime(formItem, recordNumber, entryData);

        // Now this item is added, the number for the next record should be
        // one higher
        recordNumber++;
        // TODO Fix that clicking this button will clear previous added fields
        $(recordAddButton).attr({
            'onclick': 'Splash.addSingleEntryTime("formAthleteRecordBody",'
            + recordNumber + ')'
        });
        // TODO Move this to the attr function above
        $('#recordAdd').data("numberOfRecords", recordNumber);
    },

    displayEntryTimes: function(modal, entriesObj, athleteId) {
        var loop = 0, entryData = {}, entries = [];
        var formItem = document.getElementById('formAthleteRecordBody');
        var recordAddButton = document.getElementById('recordAdd');

        // Reset the current body
        formItem.innerHTML = "<input type='hidden' class='form-control'"
            + " id='athleteAthleteId' value='" + athleteId + "'/>";

        if (!entriesObj) {
            // There is nothing to display, show an empty record field
            this.addSingleEntryTime("formAthleteRecordBody", 0);
            // We need to increase the loop to have the right value for the
            // add record button
            loop++;
        } else {
            // Make sure the entry can be handled as an array
            entries = entriesObj.ENTRY;
            entries = this.ensureArray(entries);
            // Make a loop to display ALL entry times
            for (loop=0; loop < entries.length; loop++) {
                entryData.entryTime = entries[loop]['@entrytime']
                entryData.entryName = entries[loop].SWIMSTYLE['@name']
                this.displaySingleEntryTime(formItem, loop, entryData);
            }
        }
        // Add an action to the add record button
        // XXX I don't know why recordAddButton.onclick = "" does not work
        $(recordAddButton).attr({
            'onclick': 'Splash.addSingleEntryTime("formAthleteRecordBody",'
            + loop + ')'
        });
        $('#recordAdd').data("numberOfRecords", loop);
    },

    displayAthleteEvents: function(modal, athlete) {
        var evLoop = 0, entryData = {}, entry = [];
        var formItem = document.getElementById('formAthleteCompetitionBody');
        var sesLoop = 0, ageLoop = 0, enLoop = 0, subscLoop = 0;
        var session = Splash.meetInfo.SESSIONS.SESSION;
        var eventItem, agegroups = undefined, ageGroup, ageGroupId = null;
        var validEvent = false, oneValidEvent = false;

        // Reset the current body
        formItem.innerHTML = "<input type='hidden' class='form-control'"
            + " id='athleteAthleteId' value='" + athlete.athleteId + "'/>";

        session = this.ensureArray(session);
        for (sesLoop = 0; sesLoop < session.length; sesLoop++) {
            eventItem = session[sesLoop].EVENTS.EVENT;
            eventItem = this.ensureArray(eventItem);
            for (evLoop = 0; evLoop < eventItem.length; evLoop++) {
                var currentEvent = eventItem[evLoop];
                var curEventId = currentEvent['@eventid'];
                validEvent = false;
                ageGroupId = null;
                // loop through all EVENTS and only show the ones that are
                // within the age-range for this athlete
                if (currentEvent['@gender'] == athlete.gender &&
                    currentEvent.SWIMSTYLE['@relaycount'] == "1") {
                    // TODO Move this part to a function

                    agegroups = currentEvent.AGEGROUPS
                    if (agegroups) {
                        ageGroup = agegroups.AGEGROUP;
                        ageGroup = this.ensureArray(ageGroup);
                        // Test if there is an agegroup that is good for this
                        // athlete
                        for (ageLoop = 0;
                                !validEvent && ageLoop < ageGroup.length;
                            ageLoop++)
                        {
                            var curAgeGr = ageGroup[ageLoop];
                            // An -1 for agemin or agemax means no min or max
                            if (curAgeGr['@agemin'] <= athlete.age &&
                                 (athlete.age <= curAgeGr['@agemax'] ||
                                  curAgeGr['@agemax'] == "-1"
                                 )
                               ) {
                                validEvent = true;
                                oneValidEvent = true;
                                ageGroupId = curAgeGr['@agegroupid'];
                            }
                        }
                    } else {
                        //console.log("id " + curEventId + "has all ages");
                        validEvent = true;
                        oneValidEvent = true;
                    }
                    if (validEvent) {
                        var inputEventData = document.createElement('input');
                        var input = document.createElement('input');
                        var labelEventName = document.createElement('label');
                        var select = document.createElement('select');
                        var inputTime = document.createElement('input');
                        var eventData = {};

                        $(inputEventData).attr({
                            'id': 'athleteEventData' + subscLoop,
                            'type': 'hidden',
                            'class': 'form-control input-group-sm'
                        });
                        eventData.eventId = curEventId;
                        eventData.ageGroupId = ageGroupId;
                        eventData.name = this.getEventName(
                            currentEvent.SWIMSTYLE);
                        $(input).attr({
                            'id': 'athleteCompCheckbox' + subscLoop,
                            'type': 'checkbox',
                            'class': 'form-control input-group-sm'
                        });
                        $(labelEventName).attr({
                            'id': 'divAthleteCompEventName' + subscLoop,
                            'for': 'athleteCompCheckbox' + subscLoop,
                            'class': 'control-label'
                        });
                        $(select).attr({
                            'id': 'athleteEntryTimes' + subscLoop,
                            'class': 'form-control input-group-sm'
                        });
                        // TODO When something is entered here, the item should
                        // be checked, the same for selecting a time
                        $(inputTime).attr({
                            'id': 'athleteCompTime' + subscLoop,
                            'type': 'text',
                            'placeholder': 'HH:MM:SS.ss',
                            'size': '13',
                            'class': 'form-control input-group-sm'
                        });

                        labelEventName.textContent = eventData.name + " (" + curEventId + ")";
                        formItem.appendChild(inputEventData);
                        formItem.appendChild(input);
                        formItem.appendChild(labelEventName);
                        formItem.appendChild(select);
                        formItem.appendChild(inputTime);
                        formItem.appendChild(document.createElement('br'));

                        modal.find('#athleteEventData' + subscLoop)
                            .data("eventData", eventData);

                        // Add all athletes records as option to the select
                        // with an default of NT
                        $(formItem).find('#athleteEntryTimes'
                            + subscLoop).append(
                            $("<option></option>")
                                .attr("value", "-1")
                                .text("Select Entry Time (NT)")
                        );
                        try {
                            entry = athlete.ENTRIES.ENTRY;
                            entry = this.ensureArray(entry);
                            for (enLoop = 0; enLoop < entry.length; enLoop++) {
                            // TODO Only use the SWIMSTYLE@name when the stroke
                            // is UNKNOWN
                                // We use the value of the index in the athlete
                                // records (=enLoop)
                                if (entry[enLoop]['@eventid'] == curEventId) {
                                    // The event has already been selected
                                    $(input).attr({
                                        'checked': 'checked'
                                    });
                                    $(formItem).find('#athleteEntryTimes'
                                        + subscLoop).append(
                                        $("<option></option>")
                                            .attr("value", "I-" + enLoop)
                                            .attr("selected", "selected")
                                            .text(entry[enLoop].SWIMSTYLE['@name']
                                                + "(" + entry[enLoop]['@entrytime']
                                                + ")")
                                    );
                                } else {
                                    $(formItem).find('#athleteEntryTimes'
                                        + subscLoop).append(
                                        $("<option></option>")
                                            .attr("value", "I-" + enLoop)
                                            .text(entry[enLoop].SWIMSTYLE['@name']
                                                + "(" + entry[enLoop]['@entrytime']
                                                + ")")
                                    );
                                }
                            }
                        } catch (e) {
                            // TODO Only show this message once or not at all
                            //alert("This athlete has no records");
                        }
                    }
                }
                subscLoop++;
            }
        }
        // if we didn't find a valid competition let the user know
        if (!oneValidEvent) {
            formItem.innerHTML += "No competition for "
                + this.getGenderText(athlete.gender) + " athletes of "
                + athlete.age + " years old found.";
        }
        $('#saveAthleteCompetition').data("numberOfEvents", subscLoop);
    },

    displayRelayEvents: function(modal, relayData) {
        var evLoop = 0;
        var formItem = document.getElementById('formRelayCompetitionBody');
        var sesLoop = 0, ageLoop = 0;
        var session = Splash.meetInfo.SESSIONS.SESSION;
        var input = document.createElement('input');
        var labelAthletes = document.createElement('label');
        var inputEventData = document.createElement('input');
        var select = document.createElement('select');
        var agegroups = undefined, ageGroup, ageGroupId = null;
        var eventItem, entry;
        var validEvent = false, oneValidEvent = false;
        var relay = {}, relayPos;
        var athleteIndex = 0, athlete = 0, athleteId = 0;

        // Reset the current body
        formItem.innerHTML = "<input type='hidden' class='form-control'"
            + " id='teamRelayId' value='" + relayData.id + "'/>";

        $(input).attr({
            'id': 'relayCompCheckbox',
            'type': 'checkbox',
            'class': 'form-control input-group-sm'
        });
        $(labelAthletes).attr({
            'id': 'divAthleteCompEventName',
            'for': 'relayCompCheckbox',
            'class': 'control-label'
        });

        if (relayData.eventId) {
            $(input).attr({
                'checked': 'checked'
            });
        }

        relayData.names = "";
        entry = this.club[relayData.clubId].RELAYS
                .RELAY[relayData.relayIndex].ENTRIES.ENTRY;
        if (relayData.entryIndex !== undefined && Array.isArray(entry)) {
            relayPos = entry[relayData.entryIndex].RELAYPOSITIONS.RELAYPOSITION;
        } else {
            relayPos = entry.RELAYPOSITIONS.RELAYPOSITION;
        }
        relayPos = this.ensureArray(relayPos);
        for (loop = 0; loop < relayData.nrAthletes; loop++) {
            // Create the text to display at the meet
            athleteId = relayPos[loop]['@athleteid'];
            athleteIndex = this.getAthleteIndex(athleteId, relayData.clubId);
            athlete = this.getAthleteData(relayData.clubId, athleteIndex);
            if (athlete.namePrefix) {
                relayData.names += athlete.namePrefix + " ";
            }
            relayData.names += athlete.lastName;
            if (loop != relayData.nrAthletes - 1) {
                // Only add a , when other names will be added
                relayData.names += ", ";
            }
        }
        labelAthletes.textContent = relayData.names;
        formItem.appendChild(input);
        formItem.appendChild(labelAthletes);

        $(inputEventData).attr({
            'id': 'relayEventData',
            'type': 'hidden',
            'class': 'form-control input-group-sm'
        });
        $(select).attr({
            'id': 'relayEvents',
            'class': 'form-control input-group-sm'
        });
        formItem.appendChild(inputEventData);
        formItem.appendChild(select);
        formItem.appendChild(document.createElement('br'));

        $(formItem).find('#relayEvents').append(
            $("<option></option>")
                .attr("value", "-1")
                .text("No event selected")
        );

        session = this.ensureArray(session);
        for (sesLoop = 0; sesLoop < session.length; sesLoop++) {
            eventItem = session[sesLoop].EVENTS.EVENT;
            eventItem = this.ensureArray(eventItem);
            for (evLoop = 0; evLoop < eventItem.length; evLoop++) {
                var currentEvent = eventItem[evLoop];
                var curEventId = currentEvent['@eventid'];
                validEvent = false;
                //ageGroupId = null;
                // loop through all EVENTS and only show the ones that are
                // within the age-range for this athlete
                if (currentEvent['@gender'] == relayData.gender &&
                    currentEvent.SWIMSTYLE['@relaycount'] == relayData.nrAthletes) {
                    // TODO Move this part to a function

                    //agegroups = currentEvent.AGEGROUPS
                    // TODO handle the ages for team events the correct way
                    agegroups = undefined;
                    if (agegroups) {
                        ageGroup = agegroups.AGEGROUP;
                        ageGroup = this.ensureArray(ageGroup);
                        // Test if there is an agegroup that is good for this
                        // athlete
                        for (ageLoop = 0;
                                !validEvent && ageLoop < ageGroup.length;
                            ageLoop++)
                        {
                            var curAgeGr = ageGroup[ageLoop];
                            // An -1 for agemin or agemax means no min or max
                            if (curAgeGr['@agemin'] <= relayData.age &&
                                 (relayData.age <= curAgeGr['@agemax'] ||
                                  curAgeGr['@agemax'] == "-1"
                                 )
                               ) {
                                validEvent = true;
                                oneValidEvent = true;
                                ageGroupId = curAgeGr['@agegroupid'];
                            }
                        }
                    } else {
                        //console.log("id " + curEventId + "has all ages");
                        validEvent = true;
                        oneValidEvent = true;
                    }
                    if (validEvent) {
                        var eventData = {};

                        eventData.eventId = curEventId;
                        eventData.ageGroupId = ageGroupId;
                        eventData.name = this.getEventName(
                            currentEvent.SWIMSTYLE);

                        modal.find('#relayEventData')
                            .data("eventData", eventData);

                        // Add all the possible events to the select element
                        if (relayData.eventId == eventData.eventId) {
                            $(formItem).find('#relayEvents').append(
                                $("<option></option>")
                                    .attr("value", eventData.eventId)
                                    .attr("selected", "selected")
                                    .text(eventData.name + "(" + eventData.eventId
                                        + ")")
                            );
                        } else {
                            $(formItem).find('#relayEvents').append(
                                $("<option></option>")
                                    .attr("value", eventData.eventId)
                                    .text(eventData.name + "(" + eventData.eventId
                                        + ")")
                            );
                        }
                    }
                }
            }
        }
        // if we didn't find a valid competition let the user know
        if (!oneValidEvent) {
            formItem.innerHTML += "No competition for "
                + this.getGenderText(relayData.gender) + " athletes of "
                + athlete.age + " years old found.";
        }
        //$('#saveRelayCompetition').data("numberOfEvents", subscLoop);
    },

    cleanupClubData: function(clubArr) {
        // Only delete when the entry does not have an eventid, because the
        // eventid is matched to the current competition
        var loop2 = 0, enLoop = 0, entries = [], athletes = [];
        var relays = [];
        var exportAllSwimmersId, keepSwimmers = false;

        exportAllSwimmersId = document.getElementById('exportAllSwimmers');
        keepSwimmers = $(exportAllSwimmers).prop("checked");

        if (clubArr.ATHLETES) {
            athletes = clubArr.ATHLETES.ATHLETE;
            athletes = this.ensureArray(athletes);
            for (loop2 = 0; loop2 < athletes.length; loop2++) {
                if (athletes[loop2].ENTRIES) {
                    entries = athletes[loop2].ENTRIES.ENTRY;
                    entries = this.ensureArray(entries);

                    for (enLoop = 0; enLoop < entries.length; enLoop++) {
                        if (!entries[enLoop]['@eventid']) {
                            // Remove entries[enLoop], because it is not
                            // matched
                            entries.splice(enLoop,1);
                            enLoop--;
                        } else {
                            // We still have to remove the SWIMSTYLE
                            delete entries[enLoop].SWIMSTYLE;
                        }
                    }
                    if (entries.length == 1) {
                        // Convert it to an object, because that's needed for
                        // the export to xml.
                        entries = entries[0];
                    }
                    athletes[loop2].ENTRIES.ENTRY = entries;
                    if (entries.length == 0) {
                        // If there is no data in the array, delete it
                        delete athletes[loop2].ENTRIES.ENTRY;
                    }
                    if (!athletes[loop2].ENTRIES.ENTRY) {
                        // Remove the empty ENTRIES
                        delete athletes[loop2].ENTRIES;
                        if (!keepSwimmers) {
                            // Remove the athlete because s/he has no entries
                            athletes.splice(loop2,1);
                            loop2--;
                        }
                    }
                } else if (!keepSwimmers) {
                    // Remove the athlete because s/he has no entries
                    athletes.splice(loop2,1);
                    loop2--;
                }
            }
        }

        // clearup the relays
        if (clubArr.RELAYS) {
            relays = clubArr.RELAYS.RELAY;
            relays = this.ensureArray(relays);
            for (loop2 = 0; loop2 < relays.length; loop2++) {
                if (relays[loop2].ENTRIES) {
                    entries = relays[loop2].ENTRIES.ENTRY;
                    entries = this.ensureArray(entries);

                    for (enLoop = 0; enLoop < entries.length; enLoop++) {
                        if (!entries[enLoop]['@eventid']) {
                            // Remove entries[enLoop], because it is not
                            // matched
                            entries.splice(enLoop,1);
                            enLoop--;
                        } else {
                            // We always have to remove the SWIMSTYLE
                            delete entries[enLoop].SWIMSTYLE;
                        }
                    }
                    if (entries.length == 1) {
                        // Convert it to an object, because that's needed for
                        // the export to xml.
                        entries = entries[0];
                    }
                    relays[loop2].ENTRIES.ENTRY = entries;
                    if (entries.length == 0) {
                        // If there is no data in the array, delete it
                        delete relays[loop2].ENTRIES.ENTRY;
                    }
                    if (!relays[loop2].ENTRIES.ENTRY) {
                        // If there are no entries, the relay is empty and
                        // should be removed
                        //delete relays[loop2];
                        relays.splice(loop2,1);
                        loop2--;
                    }
                }
            }
            if (relays.length == 1) {
                // Convert it to an object, because that's needed for
                // the export to xml.
                relays = relays[0];
            }
        }

        return clubArr;
    },

    getClubData: function(removeEntries) {
        // loop through all the clubs in Splash.club using _.allKeys
        // (see below) and add those as an array
        var loop, keyName, allClubs = _.allKeys(Splash.club);
        var clubArr = [];

        if (!removeEntries) {
            removeEntries = false;
        }

        // TODO This could be done with a forEach call and a function (see
        // displayClubs)
        if (allClubs.length === 1) {
            // deep clone using jQuery
            //clubArr = jQuery.extend(true, {}, Splash.club[allClubs[0]]);
            // deep clone using JSON exploit
            // (http://heyjavascript.com/4-creative-ways-to-clone-objects/)
            clubArr = (JSON.parse(JSON.stringify(Splash.club[allClubs[0]])));
            if (removeEntries === true) {
                // Remove unwanted information before exporting
                clubArr = this.cleanupClubData(clubArr);
            }
        } else {
            for (loop in allClubs) {
                keyName = allClubs[loop];
                // deep clone using JSON exploit, see above
                clubArr[loop] = (JSON.parse(JSON.stringify(Splash.club[keyName])));
                if (removeEntries === true) {
                    clubArr[loop] = this.cleanupClubData(clubArr[loop]);
                }
            }
        }
        return clubArr;
    },

    exportClubs: function() {
        var lenexXmlString = "", fileName = "";
        var splashLenexDataObj = {LENEX: Splash.constructorData};
        var xmlHeader = '<?xml version="1.0" encoding="UTF-8"?>\r\n';
        var xmlPadding = "  ";
        var aNode = document.getElementById('exportClubsLink');

        // Clear the text, because creating the data takes some time
        aNode.textContent = "";

            // TODO Also handle multiple meets in one file?
        splashLenexDataObj.LENEX.MEETS = {MEET: {CLUBS: {CLUB: []}}};
        splashLenexDataObj.LENEX.MEETS.MEET.CLUBS = {CLUB: []};
        // Get the club data from the internal structure
        splashLenexDataObj.LENEX.MEETS.MEET.CLUBS.CLUB = Splash.getClubData();
        if (splashLenexDataObj.LENEX.MEETS.MEET.CLUBS.CLUB.length > 1 ) {
            fileName = splashLenexDataObj.LENEX.MEETS.MEET.CLUBS.CLUB[0]['@name'];
            fileName += " and others";
        } else {
            fileName = splashLenexDataObj.LENEX.MEETS.MEET.CLUBS.CLUB['@name'];
        }
        fileName += ".lef";

        // TODO check if this is correct
        //for (var loop = 0; loop < this.shouldBeArray.length; loop++) {
        //    this.convertFromArray(splashLenexDataObj, this.shouldBeArray[loop]);
        //}

        lenexXmlString = xmlHeader + json2xml(splashLenexDataObj, xmlPadding);
        // http://stackoverflow.com/questions/16329293/save-json-string-to-client-pc-using-html5-api
        aNode.download = fileName;
        aNode.href = URL.createObjectURL(new Blob([lenexXmlString],
            {type: "text/xml"}));
        aNode.textContent = "download";
        this.setWarnBeforeClose(false);
    },

    exportOldCompetition: function() {
        var lenexXmlString = "", competitionName = "";
        var lenexDataObj = {LENEX: Splash.lenexDataObj};
        var xmlHeader = '<?xml version="1.0" encoding="UTF-8"?>\r\n';
        var xmlPadding = "  ";
        var aNode = document.getElementById('exportCompetitionLink');

        // Clear the text, because creating the data takes some time
        aNode.textContent = "";

        if (Splash.lenexDataObj['@version'] === "2.0") {
            xmlHeader = '<?xml version="1.0" encoding="ISO-8859-1"?>\r\n';
            xmlPadding = " ";
        }
        lenexXmlString = xmlHeader + json2xml(lenexDataObj, xmlPadding);

            // TODO Also handle multiple meets in one file?
        //competitionName = Splash.lenexDataObj.MEETS.MEET[0]['@name'];
        competitionName = Splash.lenexDataObj.MEETS.MEET['@name'];
        aNode.download = competitionName + '.lef';
        aNode.href = URL.createObjectURL(new Blob([lenexXmlString],
            {type: "text/xml"}));
        aNode.textContent = "download Original competition file";
    },

    /*
    matchEvents: function() {
        // Loop through the athletes and relays and ask which event in the
        // current competition they match (or none)
        // orgeventid -> eventid (key value pair) in processedEventIds

        // This will handle the modal
        $('#modalMatchEvents').on('show.bs.modal', function (eventData) {
            // link or button that triggered the modal
            var relation = $(eventData.relatedTarget);
            //var athleteObj = Splash.getAthleteData(clubId, athleteIndex);
            var eventName = "Test event";

            // TODO Need to see how we can loop through all the saved entries
            var orgEventId = Splash.club[Splash.currentClubIndex].ATHLETES.ATHLETE[2].ENTRIES.ENTRY[0]['@orgeventid'];
            eventName = Splash.club[Splash.currentClubIndex].ATHLETES.ATHLETE[2].ENTRIES.ENTRY[0].SWIMSTYLE['@name'];
            //eventName += " (" + Splash.club[Splash.currentClubIndex].ATHLETES.ATHLETE[2].ENTRIES.ENTRY[0].SWIMSTYLE['@gender'] + ")";

            // Update the modal's content
            var modal = $(this);
            modal.find('.modal-title').text('Which event matches: ' +
                eventName + " gender+agegroup?");
            // Add the orgEventId to the save button to be able to retreive it
            modal.find('#saveEventId').attr("data-org-event-id", orgEventId);
            // TODO Don't do this when we can save the eventid's
            //modal.find('#saveEventId').attr("disabled", "disabled");
        });
        // XXX What to do when the match is saved
        $('#saveEventId').click(function(eventData) {
            // link or button that triggered the modal
            // extract info from the data-* attributes (see
            // api.jquery.com/data/ for more information)
            var relation = $('#saveEventId').data();
            var orgEventId = relation.orgEventId;

            // Save the current match so we don't have to do this event again
            var selectedEventId = $('input[name=newEventId]:checked').val();
            Splash.processedEventIds[orgEventId] = selectedEventId;

            $('#modalMatchEvents').modal('hide');

            Splash.setWarnBeforeClose(true);
        });
    },
    */

    exportCompetition: function() {
        var lenexXmlString = "", loop=0, competitionName = "";
        var splashLenexDataObj = {LENEX: Splash.constructorData};
        var xmlHeader = '<?xml version="1.0" encoding="UTF-8"?>\r\n';
        var xmlPadding = "  ";
        var aNode = document.getElementById('exportCompetitionLink');

        // Clear the text, because creating the data takes some time
        aNode.textContent = "";

        splashLenexDataObj.LENEX.MEETS = {MEET: {CLUBS: {CLUB: []}}};
        splashLenexDataObj.LENEX.MEETS.MEET = Splash.meetInfo;
        // Create the new CLUBS object, because it has been overwritten
        splashLenexDataObj.LENEX.MEETS.MEET.CLUBS = {CLUB: []};
        // Get the club data from the internal structure
        splashLenexDataObj.LENEX.MEETS.MEET.CLUBS.CLUB =
            Splash.getClubData(true);

        // TODO check if this is correct
        //for (var loop = 0; loop < this.shouldBeArray.length; loop++) {
        //    this.convertFromArray(splashLenexDataObj, this.shouldBeArray[loop]);
        //}

        // We don't check for version 2.0, because we won't export that version
        lenexXmlString = xmlHeader + json2xml(splashLenexDataObj, xmlPadding);

            // TODO Also handle multiple meets in one file?
        //competitionName = splashLenexDataObj.LENEX.MEETS.MEET[0]['@name'];
        competitionName = splashLenexDataObj.LENEX.MEETS.MEET['@name'];
        aNode.download = competitionName + '.lef';
        aNode.href = URL.createObjectURL(new Blob([lenexXmlString],
            {type: "text/xml"}));
        aNode.textContent = "download";
        this.setWarnBeforeClose(false);
    },

    displayData: function() {
        var lenexString = "", index = 0, index1 = 0, keyName = "";
        var keyName1 = "", lenexData1 = {}, allKeys1 = "";
        var allKeys = _.allKeys(this.lenexDataObj);

        this.debugDumpNode.innerHTML = Date();

        for (index in allKeys) {
            keyName = allKeys[index];
            if (_.isObject(this.lenexDataObj[keyName])) {
                lenexString += "E: " + keyName + "<br />";

                lenexData1 = _.clone(this.lenexDataObj[keyName]);
                allKeys1    = _.allKeys(lenexData1);
                for (index1 in allKeys1) {
                    keyName1 = allKeys1[index1];
                    if (!_.isObject(lenexData1[keyName1])) {
                        lenexString += "&nbsp;1 " + keyName1 + ": " +
                            lenexData1[keyName1] + "<br />";
                    }
                }
            } else {
                lenexString += keyName + ": " + this.lenexDataObj[keyName] +
                    "<br />";
            }
        }
        this.debugDumpNode.innerHTML = lenexString;
    },

    /*
    // XXX added this function to see if it will help
    // XXX currently it only converts the first objectName to an array
    // http://stackoverflow.com/questions/5443436/how-do-i-recursively-search-an-object-tree-and-return-the-matching-object-based
    convertToArray: function(originalObj, objectName) {
        var p, key, val, tRet;
        if (originalObj.hasOwnProperty(objectName)) {
            val = originalObj[objectName] instanceof Array;
            if (!val) {
                originalObj[objectName] = [originalObj[objectName]];
            }
            return originalObj[objectName];
        }
        for (p in originalObj) {
            if (p == objectName) {
                val = originalObj[objectName] instanceof Array;
                if (!val) {
                    originalObj[objectName] = [originalObj[objectName]];
                }
                return originalObj[objectName];
            } else if (originalObj[p] instanceof Object) {
                if (originalObj.hasOwnProperty(p)) {
                    tRet = this.convertToArray(originalObj[p], objectName);
                    if (tRet) { return tRet; }
                }
            }
        }
        return false;
    },

    convertFromArray: function(originalObj, objectName) {
        var p, key, val, tRet;
        if (originalObj.hasOwnProperty(objectName)) {
            val = originalObj[objectName] instanceof Array;
            if (!val) {
                originalObj[objectName] = [originalObj[objectName]];
            }
            return originalObj[objectName];
        }
        for (p in originalObj) {
            if (p == objectName) {
                val = originalObj[objectName] instanceof Array;
                if (!val) {
                    originalObj[objectName] = [originalObj[objectName]];
                }
                return originalObj[objectName];
            } else if (originalObj[p] instanceof Object) {
                if (originalObj.hasOwnProperty(p)) {
                    tRet = this.convertFromArray(originalObj[p], objectName);
                    if (tRet) { return tRet; }
                }
            }
        }
        return false;
    },
    */

    createDataStructure: function(lenexObj, inSender) {
        // TODO create a list with all lenex items and use a loop to see if it
        // is available in the json
        // TODO maybe use an assosiative array with filename => js-object(with
        // lef data)
        // TODO when loading a club and a competiton, let the user match the
        // local eventid's with the competition eventid's
        var handleMeet = false;
        var loop = 0;

        if (inSender === 'meetFile') {
            handleMeet = true;
        }

        //for (loop = 0; loop < this.shouldBeArray.length; loop++) {
        //    this.convertToArray(lenexObj, this.shouldBeArray[loop]);
        //}

        // TODO Move this function to somewhere else
        var parseLenexData = function(data) {
            // TODO Also handle multiple meets in one file?
            //var clubs = data.MEETS.MEET[0].CLUBS;
            var clubs = data.MEETS.MEET.CLUBS;
            var sessions, session, race, dataSession;
            var loop, loop2, version2 = false;

            if (data['@version'] === '2.0') {
                // We need to handle some things differently when it is version
                // 2.0
                version2 = true;
            }

            if (handleMeet) {
                try {
            // TODO Also handle multiple meets in one file?
                    //dataSession = data.MEETS.MEET[0].SESSIONS.SESSION;
                    dataSession = data.MEETS.MEET.SESSIONS.SESSION;
                } catch (e) {
                    alert('This file does not contain sessions');
                    // TODO Handle this better
                    return;
                }
                this.sourceData = data;
                // Copy the meet info to be able to save it later, it won't
                // need club data
            // TODO Also handle multiple meets in one file?
                //this.meetInfo = data.MEETS.MEET[0];
                this.meetInfo = data.MEETS.MEET;
                delete this.meetInfo.CLUBS;

                if (!Array.isArray(dataSession)) {
                    // Create a new array to handle it the same as no array
                    sessions = [];
                    sessions[0] = dataSession;
                } else {
                    sessions = dataSession;
                }
                for (loop=0; loop<sessions.length; loop++) {
                    session = sessions[loop];
                    // XXX Why are we saving this as an object?
                    if (!Array.isArray(session.EVENTS.EVENT)) {
                        this.competition[session.EVENTS.EVENT['@eventid']] =
                            session.EVENTS.EVENT;
                    } else {
                        session.EVENTS.EVENT.forEach(function(race) {
                                this.competition[race['@eventid']] = race;
                            }, this);
                    }
                }
            }

            if (!handleMeet && clubs === undefined) {
                alert('no clubs in file');
                return;
            }
            if (clubs !== undefined && clubs.CLUB) {
                // TODO XXX copy the events after conversion
                // Save the swimstyles to a local data array, so we can use it
                // to look them up
                try {
            // TODO Also handle multiple meets in one file?
                    //dataSession = data.MEETS.MEET[0].SESSIONS.SESSION;
                    dataSession = data.MEETS.MEET.SESSIONS.SESSION;
                    if (!Array.isArray(dataSession)) {
                        // Create a new array to handle it the same as no array
                        sessions = [];
                        sessions[0] = dataSession;
                    } else {
                        sessions = dataSession;
                    }
                    for (loop=0; loop<sessions.length; loop++) {
                        session = sessions[loop];
                        if (!Array.isArray(session.EVENTS.EVENT)) {
                            this.swimstyles[session.EVENTS.EVENT['@eventid']] =
                                session.EVENTS.EVENT.SWIMSTYLE;
                            // Add the gender to the swimstyle, because it
                            // is needed for matching
                            //this.swimstyles[session.EVENTS.EVENT['@eventid']]['@gender'] =
                            //    session.EVENTS.EVENT['@gender'];
                        } else {
                            session.EVENTS.EVENT.forEach(function(race) {
                                this.swimstyles[race['@eventid']] =
                                    race.SWIMSTYLE;
                                // Add the gender to the swimstyle
                                //this.swimstyles[race['@eventid']]['@gender'] =
                                //    race['@gender'];
                                }, this);
                        }
                    }
                } catch (e) {
                    // Silently ignore the error, because the sessions are not
                    // mandatory in a club file
                    dataSession = "";
                }
                // Save the clubs to a local var in an array to be able to add
                // additional clubs when a new club file is loaded
                Splash.handleObjectOrArray(this.storeClub, clubs.CLUB, this);
            }

            return true;
        }.bind(this);

        parseLenexData(lenexObj);

        this.lenexDataObj = lenexObj;
        this.aLenexData.push(lenexObj);

        // Now display the meet information
        if (handleMeet) {
            Splash.displayMeet();
        }
        Splash.displayClubs();

        // 'print' loaded file + button to press and display
        document.getElementById('displayDataButton').innerHTML = '\
<button onclick="Splash.displayData()">show constructor</button>\
<button onclick="Splash.displayTreeData()">get Tree</button>\
<button onclick="Splash.exportOldCompetition()">OLD Competition export</button><br />';
    },

    displaySingleSession: function(oSession, index, allSessions) {
        var sessionNumber = oSession['@number'];
        var daytime = oSession['@daytime'];
        var liNode = document.createElement('li');
        var aSessionLink = document.createElement('a');

        this.ulSessionInfo = document.createElement('ul');

        $(aSessionLink).attr({
            'data-toggle': 'collapse',
            'href': '#session' + sessionNumber
        });

        aSessionLink.textContent = "("+ sessionNumber +") " + oSession['@date'];
        if (daytime) {
            aSessionLink.textContent += " " + daytime;
        }

        $(this.ulSessionInfo).attr({
            'id': 'session' + sessionNumber,
            'class': 'list-unstyled collapse'
        });

        // Add everything for all the events
        Splash.handleObjectOrArray(Splash.displaySingleEvent,
            oSession.EVENTS.EVENT, Splash);

        liNode.appendChild(aSessionLink);
        this.eventsInfoNode.insertBefore(liNode, null);
        this.eventsInfoNode.insertBefore(this.ulSessionInfo, null);
    },

    displaySingleEvent: function(oEvent, index, allEvents) {
        var gender = "", sessionText = "";
        var eventId = oEvent['@eventid'];
        var distance = oEvent.SWIMSTYLE['@distance'];
        var stroke = this.getEventName(oEvent.SWIMSTYLE);
        var nrAthletes = oEvent.SWIMSTYLE['@relaycount'];
        var swimStyleId = oEvent.SWIMSTYLE['@swimstyleid'];
        var swimStyleName = oEvent.SWIMSTYLE['@name'];
        var liEventNode = document.createElement('li');
        var ulAthletes = document.createElement('ul');
        var liAthleteAndTime = document.createElement('li');
        var liAddTeam = document.createElement('li');
        var buttonAddTeam = document.createElement('button');
        var eventAthletes = {};

        gender = this.getGenderText(oEvent['@gender']);

        if (distance) {
            if (nrAthletes > 1) {
                sessionText += nrAthletes + " x ";
            }
            sessionText += distance + "m, ";
        }
        sessionText += stroke + ", " + gender;

        // Handle the agegroup information for the event
        this.currentEventAgeGroupText = "";
        try {
            Splash.handleObjectOrArray(Splash.getAgeGroupInfo,
                oEvent.AGEGROUPS.AGEGROUP, Splash);
        } catch (e) {
            this.currentEventAgeGroupText = "all ages";
        }
        // Only show the age when there is only 1 age group (it contains no ,)
        if (this.currentEventAgeGroupText.indexOf(",") === -1) {
            sessionText += ", " + this.currentEventAgeGroupText;
        }
        sessionText += " (" + eventId + ")";

        // Create a link to make it possible to expand the event to see the
        // athletes
        var aEventLink = document.createElement('a');

        $(aEventLink).attr({
            'data-toggle': 'collapse',
            'href': '#ulEvent' + eventId
        });
        // TODO don't put the gender and event id in the link
        aEventLink.textContent = sessionText;

        // Create an li element to add the text to
        $(liEventNode).attr({
            'id': 'liEvent' + eventId,
            'title': this.currentEventAgeGroupText
        });
        //liEventNode.textContent = sessionText;
        liEventNode.appendChild(aEventLink);
        this.ulSessionInfo.appendChild(liEventNode);

        //this.ulSessionInfo.innerHTML += "<li title=\""
        //    + this.currentEventAgeGroupText + "\">" + sessionText + "</li>";

        // TODO Make an ul to add the athletes as li and make it
        // collapseable when it contains athletes
        //    'class': 'list-unstyled collapse in'
        $(ulAthletes).attr({
            'id': 'ulEvent' + eventId,
            'class': 'collapse in'
        });
        liEventNode.appendChild(ulAthletes);

        if (nrAthletes > 1) {
            // Add a button to add a relay team (uses a modal)
            $(buttonAddTeam).attr({
                'id': 'addTeamEvent' + eventId,
                'type': 'button',
                'class': 'btn btn-default btn-sm',
                'data-toggle': 'modal',
                'data-target': '#modalRelayInfo'
            });
            // Add data to the button to be able to get the right event
            $(buttonAddTeam).data("eventData", {"text": sessionText,
                "eventId": eventId, "nrAthletes": nrAthletes,
                "gender": oEvent['@gender']});
            buttonAddTeam.textContent = "Add team";
            liAddTeam.appendChild(buttonAddTeam);
            ulAthletes.appendChild(liAddTeam);
        }

/*
        // The data-* items are in the dataset array (dataset['toggle'] =
        // 'collapse')
        $(meetButton).attr({
            'id': 'meetButton',
            'type': 'button',
            'class': 'btn btn-info',
            'data-toggle': 'collapse',
            'data-target': '#competitionInfo'
        });

        var aSessionLink = document.createElement('a');

        $(aSessionLink).attr({
            'data-toggle': 'collapse',
            'href': '#session' + sessionNumber
        });
        var liEventNode = document.createElement('li');
        var ulAthletes = document.createElement('ul');
        var liAthleteAndTime = document.createElement('li');
*/
        // TODO Add a list of athletes that are subscribed to the event
        // loop through all athletes (this.club.???)
        eventAthletes = this.getAthletesNamesAndTimes(eventId);
        if (eventAthletes) {
            this.displayAthleteAtEvent(eventAthletes);
            //liEventNode.text = eventAthletes.name + eventAthletes.time
        }

        // Also add radio buttons to the modalMatchEvents
        var inputEventId = document.createElement('input');
        var labelNewEvent = document.createElement('label');
        inputEventId.id = "eventId" + eventId;
        inputEventId.type = "radio";
        inputEventId.name = "newEventId";
        inputEventId.value = eventId;

        labelNewEvent.className = "radio";
        labelNewEvent.appendChild(inputEventId);
        // TODO get the right age groups
        sessionText = stroke + ", " + gender + ", " +
            this.currentEventAgeGroupText;
        labelNewEvent.innerHTML += sessionText;

        document.getElementById('modalMatchEventRadios').insertBefore(labelNewEvent, null);
    },

    getAthletesNamesAndTimes: function(eventId) {
        // XXX Debug this function and fix the fixed texts
        var loop = 0;
        var athlete = {};
        if (eventId == "1058") {
            athlete.eventId = eventId;
            athlete.name = "Pietje Puk";
            athlete.entryTime = "00:01:02.03";
        } else {
            athlete = undefined;
        }

        return athlete;

/*
        var arrAthlete = this.club['Lifesaving club 30'].ATHLETES.ATHLETE;

        for (loop = 0; loop < arrAthlete.length; loop++) {
            curEventId = arrAthlete[loop].ENTRIES.ENTRY[loop2]['@eventid'];
            if (curEventId == eventId) {
                athlete.eventId = curEventId;
                athlete.name = arrAthlete[loop]['@firstname']; // TODO Use complete name
                athlete.entryTime = arrAthlete[loop].ENTRIES.ENTRY[loop2]['@entrytime'];
            }
        }

        //return object with name and time
        return athlete;
        */
    },

    displayAthleteAtEvent: function(eventAthletes) {
        var athleteId = "";

        // TODO eventAthletes can contain an array?
        if (!eventAthletes) {
            return;
        }
        // This will be unique to see if the athlete is already displayed
        athleteId = eventAthletes.eventId + "_" + eventAthletes.id;

        // TODO This does not work when loading a meet, because the ulEvent is not yet in the html
        var ulEventId = document.getElementById('ulEvent' + eventAthletes.eventId);
        var liAthlete = document.createElement('li');
        var athleteEventId = document.getElementById(athleteId);

        if (ulEventId && !athleteEventId) {
            liAthlete.textContent = eventAthletes.name + " (" + eventAthletes.entryTime + ")";
            liAthlete.id = athleteId;

            ulEventId.appendChild(liAthlete);
        }
        if (athleteEventId && eventAthletes.remove) {
            // Remove the athlete from the page
            ulEventId.removeChild(athleteEventId);
        }
    },

    displayTeamAtEvent: function(teamData) {
        var relayId = "";

        // TODO teamData can contain an array?
        if (!teamData) {
            return;
        }
        // This will be unique to see if the team is already displayed
        relayId = teamData.eventId + "_" + teamData.id;

        // TODO This does not work when loading a meet, because the ulEvent is
        // not yet in the html
        var ulEventId = document.getElementById('ulEvent' + teamData.eventId);
        var liTeam = document.createElement('li');
        var teamEventId = document.getElementById(relayId);

        if (ulEventId && teamEventId) {
            // When both items are available, we need to remove it to update
            // the entrytime
            ulEventId.removeChild(teamEventId);
            teamEventId = undefined;
        }
        if (ulEventId && !teamEventId && !teamData.remove) {
            liTeam.textContent = teamData.names + " (" + teamData.entryTime
                + ")";
            liTeam.id = relayId;

            ulEventId.appendChild(liTeam);
        }
    },

    getAgeGroupInfo: function(oAgeGroup, index, allAgeGroups) {
        // mandatory fields
        var agegroupid = oAgeGroup['@agegroupid'];
        var agemax = oAgeGroup['@agemax'];
        var agemin = oAgeGroup['@agemin'];
        // optional fields
        var gender = oAgeGroup['@gender'];
        var calculate = oAgeGroup['@calculate'];
        var handicap = oAgeGroup['@handicap'];
        var levelmax = oAgeGroup['@levelmax'];
        var levelmin = oAgeGroup['@levelmin'];
        var levels = oAgeGroup['@levels'];
        var name = oAgeGroup['@name'];
        var totalText = "";

        // add a comma when there is already a agegroup text
        if (this.currentEventAgeGroupText) {
            totalText += ", ";
        }
        if (name) {
            totalText += name + " (";
        }
        if (agemin === "-1" && agemax === "-1") {
            totalText += "all ages";
        } else if (agemin === "-1") {
            totalText += agemax + " and less";
        } else if (agemax === "-1") {
            totalText += agemin + " and more";
        } else {
            totalText += agemin + "-" + agemax + " years";
        }
        if (name) {
            totalText += ")";
        }
        this.currentEventAgeGroupText += totalText;
    },

    displayMeet: function() {
        //var contactInfo, divContact;
        var sessions, meet = Splash.meetInfo;
        var checkbox = document.createElement('input');
        var labelCheckbox = document.createElement('label');
        var divCompetitionInfo = document.createElement('div');
        var meetButton = document.createElement('button');
        var ulEventsInfo = document.createElement('ul');
        var eventsInfoID = "EventsInfo";

        ulEventsInfo.id = eventsInfoID;
        // Maybe we want the next line
        //ulEventsInfo.className = "list-unstyled";

        // This is a jquery solution
        // The data-* items are in the dataset array (dataset['toggle'] =
        // 'collapse')
        $(meetButton).attr({
            'id': 'meetButton',
            'type': 'button',
            'class': 'btn btn-info',
            'data-toggle': 'collapse',
            'data-target': '#competitionInfo'
        });
        meetButton.textContent = meet['@name'];

        $(checkbox).attr({
            'id': 'exportAllSwimmers',
            'type': 'checkbox',
            'class': 'input-group-sm'
        });
        $(labelCheckbox).attr({
            'id': 'divExportAllSwimmers',
            'for': 'exportAllSwimmers',
            'class': 'control-label'
        });
        labelCheckbox.textContent = "Export all swimmers ";


        // TODO Loop through all the items in the meet and only add them when
        // they have a value (and are useful)
        // course can have the following values: LCM, SCM, SCY, SCM16, SCM20,
        // SCM33, SCY20, SCY27, SCY33, SCY36, OPEN
        // LCM=50meters, SCM=25meters, SCY=25yards
        divCompetitionInfo.innerHTML = [
                  'City: ',           meet['@city'],
            '<br />Name: ',           meet['@name'],
            '<br />Course: ',         meet['@course'],
            '<br />Country: ',        this.getNation(meet['@nation']),
            '<br />Organisation: ',   meet['@organizer'],
            '<br />Organisation URL: ', meet['@organizer.url'],
            '<br />Results: ',        meet['@result.url'],
            '<br />Timing: ',         meet['@timing']
        ].join('');
        divCompetitionInfo.id = "competitionInfo";
        divCompetitionInfo.className = "collapse";
        // Replace the items when they are available
        try {
            document.getElementById("meetButton").textContent =
                meetButton.textContent;
            document.getElementById("competitionInfo").innerHTML =
                divCompetitionInfo.innerHTML;
            document.getElementById(eventsInfoID).innerHTML = "";
        } catch (e) {
            // Add the items when they are not available
            this.meetInfoNode.insertBefore(meetButton, null);
            this.meetInfoNode.insertBefore(divCompetitionInfo, null);
            this.meetInfoNode.insertBefore(ulEventsInfo, null);
            this.eventsInfoNode = document.getElementById(eventsInfoID);
        }
        divCompetitionInfo.appendChild(document.createElement('br'));
        divCompetitionInfo.appendChild(labelCheckbox);
        divCompetitionInfo.appendChild(checkbox);
        sessions = meet.SESSIONS;

        Splash.handleObjectOrArray(Splash.displaySingleSession,
            sessions.SESSION, Splash);

        document.getElementById('exportCompetition').innerHTML = '\
<button class="btn btn-default btn-sm" onclick="Splash.exportCompetition()">Export</button>';
    },

    displaySingleAthlete: function(oAthlete, index, allAthletes) {
        var athleteButtonID = "athleteButton" + index;
        var athleteButton = document.createElement('button');
        // Add a div (per athlete) that has the dropdown class
        var divAthleteInfo = document.createElement('div');
        var ulNode = document.createElement('ul');
        var liNode = document.createElement('li');

        var athleteObj = this.getAthleteData(this.currentClubIndex, index);

        divAthleteInfo.className = "dropdown";

        // TODO Create the button with the right values
        $(athleteButton).attr({
            // TODO use a right id, it might depend on the club #
            'id': athleteButtonID,
            'type': 'button',
            'class': 'btn btn-default btn-xs dropdown-toggle',
            'data-toggle': 'dropdown',
            'aria-haspopup': 'true',
            'aria-expanded': 'true'
        });
        athleteButton.innerHTML = athleteObj.athleteName +
            "<span class=\"caret\"></span>";

        divAthleteInfo.appendChild(athleteButton);

        $(ulNode).attr({
            'class': 'dropdown-menu',
            'aria-labelledby': athleteButtonID
        });
        // Create the menu items
        // TODO add a link for deleting the athlete, with confirmation modal
        // and maybe only set the item to deleted and add an undo button
        // TODO This should all be seperate LI items
        liNode.innerHTML = '<a href="#" data-toggle="modal" ' +
            'data-target="#modalAthleteInfo" data-club-id="' +
            this.currentClubIndex + '" data-athlete-index="' +
            index + '">Edit</a>';
        liNode.innerHTML += '<a href="#" data-toggle="modal" ' +
            'data-target="#modalAthleteRecord" data-club-id="' +
            this.currentClubIndex + '" data-athlete-index="' +
            index + '">Records</a>';
        liNode.innerHTML += '<a href="#" data-toggle="modal" ' +
            'data-target="#modalAthleteCompetition" data-club-id="' +
            this.currentClubIndex + '" data-athlete-index="' +
            index + '">Competition</a>';
        liNode.innerHTML += '<li class="divider"></li>';
        liNode.innerHTML += '<li>Delete (TODO)</li>';

        ulNode.appendChild(liNode);
        divAthleteInfo.appendChild(ulNode);

        this.athleteInfoNode.appendChild(divAthleteInfo);
    },

    displaySingleRelayEntry: function(oEvent, entryIndex, allEvents) {
        var relayIndex = this.currentRelayIndex;
        var relayButtonID = "relayButton" + relayIndex;
        var relayButton = document.createElement('button');
        // Add a div (per relay) that has the dropdown class
        var divRelayInfo = document.createElement('div');
        var ulNode = document.createElement('ul');
        var liNode = document.createElement('li');
        var aEdit = document.createElement('a');
        var aCompetition = document.createElement('a');
        var relayName = "";
        var eventId = "", nrAthletes = 0, entryTime = "";
        var eventData = {};

        var relayObj = this.getRelayData(this.currentClubIndex, relayIndex,
            entryIndex);

        if (relayObj.gender) {
            relayName += this.getGenderText(relayObj.gender) + ", ";
        }
        relayName += relayObj.relayName;

        if (oEvent) {
            eventId = oEvent['@eventid'];
            entryTime = oEvent['@entrytime'];
            nrAthletes = oEvent.RELAYPOSITIONS.RELAYPOSITION.length;
        }

        divRelayInfo.className = "dropdown";

        // TODO Create the button with the right values
        $(relayButton).attr({
            // TODO use a right id, it might depend on the club #
            'id': relayButtonID,
            'type': 'button',
            'class': 'btn btn-default btn-xs dropdown-toggle',
            'data-toggle': 'dropdown',
            'aria-haspopup': 'true',
            'aria-expanded': 'true'
        });
        relayButton.innerHTML = relayName + "<span class=\"caret\"></span>";

        divRelayInfo.appendChild(relayButton);

        $(ulNode).attr({
            'class': 'dropdown-menu',
            'aria-labelledby': relayButtonID
        });
        // Create the menu items
        $(aEdit).attr({
            'href': "#",
            'data-toggle': "modal",
            'data-target': "#modalRelayInfo",
            // XXX Add these items a single items, because an object can't be
            // transferred via a data attribute
            'data-text': relayName,
            'data-event-id': eventId,
            'data-nr-athletes': nrAthletes,
            'data-entry-time': entryTime,
            'data-club-id': this.currentClubIndex,
            'data-relay-index': relayIndex,
            'data-entry-index': entryIndex,
            'data-relay-id': relayObj.relayId,
            'data-team-number': relayObj.teamNumber,
            'data-gender': relayObj.gender,
        });
        // TODO add a link for deleting the relay, with confirmation modal
        // and maybe only set the item to deleted and add an undo button
        // TODO This should all be seperate LI items
        aEdit.textContent = "Edit Team";
        //liNode.innerHTML = '<a href="#" data-toggle="modal" ' +
        //    'data-target="#modalRelayInfo" data-club-id="' +
        //    this.currentClubIndex + '" data-relay-index="' +
        //    index + '">Edit</a>';
        liNode.appendChild(aEdit);
        //liNode.innerHTML += '<a href="#" data-toggle="modal" ' +
        //    'data-target="#modalRelayCompetition" data-club-id="' +
        //    this.currentClubIndex + '" data-relay-index="' +
        //    index + '">Competition</a>';
        $(aCompetition).attr({
            'href': "#",
            'data-toggle': "modal",
            'data-target': "#modalRelayCompetition",
            // XXX Add these items a single items, because an object can't be
            // transferred via a data attribute
            'data-text': relayName,
            'data-event-id': eventId,
            'data-nr-athletes': nrAthletes,
            'data-entry-time': entryTime,
            'data-club-id': this.currentClubIndex,
            'data-relay-index': relayIndex,
            'data-entry-index': entryIndex,
            'data-relay-id': relayObj.relayId,
            'data-team-number': relayObj.teamNumber,
            'data-gender': relayObj.gender,
        });
        aCompetition.textContent = "Competition";
        liNode.appendChild(aCompetition);
        liNode.innerHTML += '<li class="divider"></li>';
        liNode.innerHTML += '<li>Delete (TODO)</li>';

        ulNode.appendChild(liNode);
        divRelayInfo.appendChild(ulNode);
        // TODO Add a text with the athlete names
        // Need to create the list of names
        //divRelayInfo.innerHTML += "<div>" + relayObj.names + "</div>";

        this.relayInfoNode.appendChild(divRelayInfo);
    },

    displaySingleRelay: function(oRelay, index, allRelays) {
        var relayObj = this.getRelayData(this.currentClubIndex, index);

        // Handle the ENTRY because it can be an array
        if (relayObj.ENTRIES) {
            this.currentRelayIndex = index;
            this.handleObjectOrArray(this.displaySingleRelayEntry,
                relayObj.ENTRIES.ENTRY, this);
        }
    },

    saveAthleteRecord: function(clubId, athleteIndex, recordInfo) {
        var athletes = this.club[clubId].ATHLETES;
        var entries, entry;

        if (!Array.isArray(athletes.ATHLETE)) {
            // TODO do things when there is only 1 athlete
            alert("Saving records for 1 athlete is not yet supported");
        } else {
            entries = athletes.ATHLETE[athleteIndex].ENTRIES;
            if (!entries) {
                // There are no entries for this athlete yet
                athletes.ATHLETE[athleteIndex].ENTRIES = {'ENTRY': {}};
                entries = athletes.ATHLETE[athleteIndex].ENTRIES;
            }
            // There is only 1 entry, but if the index is 1, we need to
            // convert it to an array
            if (recordInfo.index == 1 && !Array.isArray(entries.ENTRY)) {
                entries.ENTRY = [entries.ENTRY];
            }
            // There is a new time for the event and there is already 1 entry
            if (recordInfo.index === undefined
                && !Array.isArray(entries.ENTRY)
                && entries.ENTRY['@entrytime']) {
                entries.ENTRY = [entries.ENTRY];
            }
            if (!Array.isArray(entries.ENTRY)) {
                entry = entries.ENTRY;
            } else {
                if (entries.ENTRY[recordInfo.index] === undefined) {
                    // If this is a new entry, we create the object for it at
                    // the end of the current array
                    recordInfo.index = entries.ENTRY.length;
                    entries.ENTRY.push({});
                }
                entry = entries.ENTRY[recordInfo.index];
            }
            if (recordInfo.name === undefined
                || recordInfo.time === undefined) {
                // Remove the record when no data has been put in either fields
                if (!Array.isArray(entries.ENTRY)) {
                    delete entries.ENTRY;
                } else {
                    entries.ENTRY.splice(recordInfo.index,1);
                }
                return false;
            }

            if (recordInfo.time.indexOf("I-") !== 0) {
                // If the time has "I-" at the start, it was an index, so we
                // don't overwrite it
                entry['@entrytime'] = recordInfo.time;
            }
            if (!entry.SWIMSTYLE) {
                // Create the object when it does not exist
                entry.SWIMSTYLE = {};
            }
            entry.SWIMSTYLE['@name'] = recordInfo.name;
            // If we have the eventId, it will be a competition entry
            if (recordInfo.eventId) {
                if (recordInfo.remove) {
                    // Remove the eventid from the entry to remove it from the
                    // subscription
                    entry['@orgeventid'] = recordInfo.eventId;
                    delete entry['@eventid'];
                    // TODO Remove the athlete from the competition list
                } else {
                    entry['@eventid'] = recordInfo.eventId;
                    delete entry['@orgeventid'];
                    // XXX For now we remove the meetinfo, because it will
                    // probably be wrong
                    delete entry.MEETINFO;

                    entry['@agegroupid'] = recordInfo.ageGroupId;
                }
            }
        }
        return true;
    },

    saveAthlete: function(clubId, athleteObj) {
        var athleteId = undefined, athletes, index = 0;
        if (!clubId) {
            return undefined;
        }
        // determine the new athleteId
        if (!Splash.club[clubId].ATHLETES) {
            // Create the athlete structure when none exist
            Splash.club[clubId].ATHLETES = {ATHLETE: []};
        }
        athletes = Splash.club[clubId].ATHLETES;
        index = athletes.ATHLETE.length;
        // Only create a new athleteid when there is none at the moment
        if (athleteObj.athleteIndex === undefined) {
            // See what the last athleteid is
            if (index) {
                athleteObj.athleteId = parseInt(athletes
                    .ATHLETE[index-1]['@athleteid']) + 1;
                // TODO check it against all the athleteid's
            } else {
                index = 0;
                athleteObj.athleteId = 0;
            }
            // Create a new athlete in the array
            athletes.ATHLETE[index] = {};
        } else {
            // Use the given index
            index = athleteObj.athleteIndex;
        }
        if (!Array.isArray(athletes.ATHLETE)) {
            // XXX Maybe do the assignment in a function, so we don't have to
            // write it twice OR use the same method as for displaying, create
            // an array and remove it when there is only 1 item
            // if the ATHLETES is available, there might be only 1 athlete,
            // which is not an array
            athletes.ATHLETE['@athleteid'] = athleteObj.athleteId;
            // TODO bithdate should be a JS Date object?
            // TODO We should remove the text after the 'T'.
            //athletes.ATHLETE['@birthdate'] = athleteObj.birthdate.toISOString();
            athletes.ATHLETE['@birthdate'] = athleteObj.birthdate;
            athletes.ATHLETE['@firstname'] = athleteObj.firstName;
            athletes.ATHLETE['@firstname.en'] = athleteObj.firstNameEn;
            athletes.ATHLETE['@gender'] = athleteObj.gender;
            athletes.ATHLETE['@lastname'] = athleteObj.lastName;
            athletes.ATHLETE['@lastname.en'] = athleteObj.lastNameEn;
            athletes.ATHLETE['@level'] = athleteObj.level;
            athletes.ATHLETE['@license'] = athleteObj.license;
            athletes.ATHLETE['@nameprefix'] = athleteObj.namePrefix;
            athletes.ATHLETE['@nation'] = athleteObj.nation;
            athletes.ATHLETE['@passport'] = athleteObj.passport;
            athletes.ATHLETE['@swrid'] = athleteObj.swrid;
        } else {
            athletes.ATHLETE[index]['@athleteid'] = athleteObj.athleteId;
            athletes.ATHLETE[index]['@birthdate'] = athleteObj.birthdate;
            athletes.ATHLETE[index]['@firstname'] = athleteObj.firstName;
            athletes.ATHLETE[index]['@firstname.en'] = athleteObj.firstNameEn;
            athletes.ATHLETE[index]['@gender'] = athleteObj.gender;
            athletes.ATHLETE[index]['@lastname'] = athleteObj.lastName;
            athletes.ATHLETE[index]['@lastname.en'] = athleteObj.lastNameEn;
            athletes.ATHLETE[index]['@level'] = athleteObj.level;
            athletes.ATHLETE[index]['@license'] = athleteObj.license;
            athletes.ATHLETE[index]['@nameprefix'] = athleteObj.namePrefix;
            athletes.ATHLETE[index]['@nation'] = athleteObj.nation;
            athletes.ATHLETE[index]['@passport'] = athleteObj.passport;
            athletes.ATHLETE[index]['@swrid'] = athleteObj.swrid;
        }
    },

    getAthleteIndex: function(athleteId, clubId) {
        var athleteIndex = undefined;
        var athleteArr = undefined;
        var loop = 0;

        if (!athleteId) {
            return undefined;
        }
        if (!clubId) {
            // TODO Loop through all the available clubs to find the athlete
            return undefined;
        } else {
            athleteArr = this.club[clubId].ATHLETES.ATHLETE;
            if (!Array.isArray(athleteArr)) {
                athleteIndex = 0;
            } else {
                for (loop = 0; loop < athleteArr.length; loop++) {
                    if (athleteArr[loop]['@athleteid'] == athleteId) {
                        athleteIndex = loop;
                    }
                }
            }
        }
        return athleteIndex;
    },

    getAthleteData: function(clubId, athleteIndex) {
        var oAthlete = {}, athleteData = {}, athleteName = "";
        var athleteArr = undefined;

        if (!clubId) {
            return undefined;
        }
        if (athleteIndex === undefined) {
            return undefined;
        }
        athleteArr = this.club[clubId].ATHLETES.ATHLETE;
        if (!Array.isArray(athleteArr)) {
            oAthlete = athleteArr;
        } else {
            oAthlete = athleteArr[athleteIndex];
        }
        athleteData = {
            athleteId: oAthlete['@athleteid'],
            birthdate: oAthlete['@birthdate'],
            firstName: oAthlete['@firstname'],
            firstNameEn: oAthlete['@firstname.en'],
            gender: oAthlete['@gender'],
            lastName: oAthlete['@lastname'],
            lastNameEn: oAthlete['@lastname.en'],
            level: oAthlete['@level'],
            license: oAthlete['@license'],
            namePrefix: oAthlete['@nameprefix'],
            // Don't convert this nation, because it is not displayed here
            nation: oAthlete['@nation'],
            passport: oAthlete['@passport'],
            swrid: oAthlete['@swrid'],
            // TODO Or should it be copied?
            ENTRIES: oAthlete['ENTRIES'],

            athleteName: ""
        };

        athleteName = athleteData.firstName;
        if (athleteData.namePrefix) {
            athleteName += " " + athleteData.namePrefix;
        }
        athleteName += " " + athleteData.lastName + " (" +
            athleteData.license + ")";

        athleteData.athleteName = athleteName;

        return athleteData;
    },

    getRelayData: function(clubId, relayIndex, entryIndex) {
        var oRelay = {}, relayData = {}, relayName = "";
        var relayArr = undefined, entry;

        if (!clubId) {
            return undefined;
        }
        if (relayIndex === undefined) {
            return undefined;
        }
        relayArr = this.club[clubId].RELAYS.RELAY;
        if (!Array.isArray(relayArr)) {
            oRelay = relayArr;
        } else {
            oRelay = relayArr[relayIndex];
        }
        relayData = {
            relayId: oRelay['@relayid'],
            agemax: oRelay['@agemax'],
            agemin: oRelay['@agemin'],
            agetotalmax: oRelay['@agetotalmax'],
            agetotalmin: oRelay['@agetotalmin'],
            gender: oRelay['@gender'],
            teamName: oRelay['@name'],
            teamNumber: oRelay['@number'],
            handicap: oRelay['@handicap'],
            // TODO Or should it be copied?
            ENTRIES: oRelay['ENTRIES'],

            relayName: ""
        };

        entry = oRelay.ENTRIES.ENTRY;
        if (entryIndex !== undefined && Array.isArray(entry)) {
            relayName += this.getEventName(entry[entryIndex].SWIMSTYLE);
        } else {
            relayName += this.getEventName(entry.SWIMSTYLE);
        }
        // Also add the team number
        relayName += " (team: " + relayData.teamNumber + ")";

        relayData.relayName = relayName;

        return relayData;
    },

    getRelayAthleteId: function(eventDataObj, athleteIndex) {
        var currentAthlete = {};
        var clubId = eventDataObj.clubId;
        var relayIndex = eventDataObj.relayIndex;
        var entryIndex = eventDataObj.entryIndex;
        var entry;
        var relayData = this.getRelayData(clubId, relayIndex, entryIndex);

        if (relayData) {
            entry = relayData.ENTRIES.ENTRY;
            if (entryIndex !== undefined && Array.isArray(entry)) {
                currentAthlete = entry[entryIndex].RELAYPOSITIONS
                    .RELAYPOSITION[athleteIndex];
            } else {
                currentAthlete = entry.RELAYPOSITIONS
                    .RELAYPOSITION[athleteIndex];
            }
            return currentAthlete['@athleteid'];
        }
        return undefined;
    },

    saveRelayTeam: function(clubId, relayData) {
        var loop = 0, posLoop = 0;
        var relay = {};
        var relayEntry = {};
        var relayPos = 0, newPos = 0, posAthleteId = 0;
        var relayPosArr = {};

        // Make sure the object exists (it might not yet exist when there are
        // no relays for this club
        if (!this.club[clubId].RELAYS) {
            this.club[clubId].RELAYS = {RELAY: {}};
        } else {
            // There are relays, but it has to be an array to add an item to it
            this.club[clubId].RELAYS.RELAY = this.ensureArray(
                this.club[clubId].RELAYS.RELAY);
        }
        // Handle when there is 1 relay (or the first relay)
        if (relayData.relayIndex === undefined) {
            // It is a new relay team that is added
            relayData.relayIndex = this.club[clubId].RELAYS.RELAY.length;
            if (relayData.relayIndex !== undefined) {
                // There is an array, so add an items to it
                this.club[clubId].RELAYS.RELAY.push({});
            }
        }

        if (Array.isArray(this.club[clubId].RELAYS.RELAY)) {
            relay = this.club[clubId].RELAYS.RELAY[relayData.relayIndex];
            // TODO Check this relayid is unique
            if (!relayData.id) {
                relayData.id = relayData.relayIndex + 1;
            }
        } else {
            relay = this.club[clubId].RELAYS.RELAY;
            // TODO Check this relayid is unique
            relayData.id = 1;
        }
        relay['@number'] = relayData.teamNumber;
        relay['@gender'] = relayData.gender;
        relay['@relayid'] = relayData.id;

        // Make sure the entry object exists
        if (!relay.ENTRIES) {
            // Make sure correct objects exists
            relay.ENTRIES = {ENTRY: {RELAYPOSITIONS: {RELAYPOSITION: [] }}};
        }
        if (relayData.entryIndex !== undefined
            && Array.isArray(relay.ENTRIES.ENTRY)) {
            relayEntry = relay.ENTRIES.ENTRY[relayData.entryIndex];
        } else {
            relayEntry = relay.ENTRIES.ENTRY;
        }
        relayEntry['@entrytime'] = relayData.entryTime;
        if (relayData.eventId && relayData.eventId != "-1") {
            relayEntry['@eventid'] = relayData.eventId;
            // Add the swimstyle, to be able to display it at the club
            relayEntry.SWIMSTYLE = this.competition[
                relayData.eventId].SWIMSTYLE;
        }
        if (relayData.remove) {
            relayEntry['@eventid'] = undefined;
            return; // We don't need to continue
        }
        relayPosArr = relayEntry.RELAYPOSITIONS.RELAYPOSITION;
        // Only save the athletes at the posisitions when they are available
        // This is not available when saving the competition entry
        if (relayData.positions) {
            for (loop = 0; loop < relayData.nrAthletes; loop++) {
                // Create a new empty object in the array when it does not yet
                // exist
                if (relayPosArr[loop] === undefined) {
                    relayPosArr.push({});
                }
                newPos = relayData.positions[loop].number;
                posAthleteId = relayData.positions[loop].athleteId;
                // Loop through all the items to find the right number, to keep the
                // positions in order. Because it will keep handling the order
                // simple
                for (posLoop = 1; posLoop <= relayData.nrAthletes; posLoop++) {
                    relayPos = relayPosArr[posLoop - 1];
                    if (parseInt(newPos) === posLoop) {
                        relayPos['@number'] = newPos;
                        relayPos['@athleteid'] = posAthleteId;
                    }
                }
            }
        }
    },

    getClubsForRelay: function(clubId) {
        //var clubs = [];
        var nrClubs = 0, clubLoop = 0;
        var select = document.createElement("select");
        var allClubs = this.getClubData(true);

        allClubs = this.ensureArray(allClubs);

        if (allClubs.length === 0) {
            return undefined;
        } else {
            nrClubs = allClubs.length;
        }

        for (clubLoop = 0; clubLoop < nrClubs; clubLoop++) {
            var option = document.createElement("option");
            curClubId = allClubs[clubLoop]['@name'];
            option.value = curClubId;
            option.textContent = curClubId;
            if (clubId === curClubId) {
                option.selected = true;
            }
            select.appendChild(option);
        }
        return select;
    },

    getAthletesForRelay: function(gender, athleteId) {
        var select = document.createElement("select");
        var option1 = document.createElement("option");
        var nrClubs = 0, nrAthletes = 0;
        var clubLoop, loop;
        var allClubs = this.getClubData();
        var athlete, clubId, athleteData = {};

        allClubs = this.ensureArray(allClubs);

        if (allClubs.length === 0) {
            return undefined;
        } else {
            nrClubs = allClubs.length;
        }
        option1.value = "-1";
        option1.textContent = "No swimmer selected";
        select.appendChild(option1);

        for (clubLoop = 0; clubLoop < nrClubs; clubLoop++) {
            var optgroup = document.createElement("optgroup");
            clubId = allClubs[clubLoop]['@name'];
            optgroup.label = clubId;
            try {
                nrAthletes = allClubs[clubLoop].ATHLETES.ATHLETE.length;
                if (nrAthletes === undefined) {
                    // There is only 1 athlete
                        var option = document.createElement("option");
                        athlete = allClubs[clubLoop].ATHLETES.ATHLETE;
                        if (athlete['@gender'] === gender
                            || gender === GENDER_MIXED
                            || gender === GENDER_ALL) {
                            // XXX This takes a lot of time?
                            athleteData = this.getAthleteData(clubId, 0);
                            if (athleteId === athlete['@athleteid']) {
                                option.selected = true;
                            }
                            option.value = athleteData.athleteId;
                            option.textContent = athleteData.athleteName;
                            optgroup.appendChild(option);
                        }
                } else {
                    for (loop = 0; loop < nrAthletes; loop++) {
                        var option = document.createElement("option");
                        athlete = allClubs[clubLoop].ATHLETES.ATHLETE[loop];
                        if (athlete['@gender'] === gender
                            || gender === GENDER_MIXED
                            || gender === GENDER_ALL) {
                            // XXX This takes a lot of time?
                            athleteData = this.getAthleteData(clubId, loop);
                            if (athleteId === athlete['@athleteid']) {
                                option.selected = true;
                            }
                            option.value = athleteData.athleteId;
                            option.textContent = athleteData.athleteName;
                            optgroup.appendChild(option);
                        }
                    }
                }
            } catch(e) {
                // There might be clubs without athletes, which we ignore
            }
            select.appendChild(optgroup);
        }

        return select;
    },

    displaySingleClub: function(oClub, index, allClubs) {
        var athletes, relays, regionText = "";
        var divClubInfo = document.createElement('div');
        var clubInfoID = "clubInfo" + index;
        var clubButtonID = "clubButton" + index;
        var athleteButtonID = "athleteButton" + index;
        var relayButtonID = "relayButton" + index;
        var clubButton = document.createElement('button');
        var clubEditButton = document.createElement('button');
        var athleteAddButton = document.createElement('button');
        var relayAddButton = document.createElement('button');
        var athleteButton = document.createElement('button');
        var relayButton = document.createElement('button');
        var divAthleteInfo = document.createElement('div');
        var divAthleteInfoID = "AthleteInfo-club" + index;
        var divAllAthletes = document.createElement('div');
        var divAllAthletesID = "AllAthletesClub" + index;
        var divRelayInfo = document.createElement('div');
        var divRelayInfoID = "RelayInfo-club" + index;
        var divAllRelays = document.createElement('div');
        var divAllRelaysID = "AllRelaysClub" + index;

        divAthleteInfo.id = divAthleteInfoID;
        divAthleteInfo.className = "btn-group-vertical";

        divAllAthletes.id = divAllAthletesID;
        // XXX Maybe make it visible by default? (by adding " in")
        divAllAthletes.className = "collapse";

        divRelayInfo.id = divRelayInfoID;
        divRelayInfo.className = "btn-group-vertical";

        divAllRelays.id = divAllRelaysID;
        // XXX Maybe make it visible by default? (by adding " in")
        divAllRelays.className = "collapse";

        // This is a jquery solution
        // The data-* items are in the dataset array (dataset['toggle'] =
        // 'collapse')
        $(clubButton).attr({
            'id': clubButtonID,
            'type': 'button',
            'class': 'btn btn-info',
            'data-toggle': 'collapse',
            'data-tooltip': 'tooltip',
            'title': 'click to view details',
            'data-target': '#' + clubInfoID
        });
        clubButton.textContent = oClub['@name'];

        $(clubEditButton).attr({
            'id': "edit" + clubButtonID,
            'type': 'button',
            'class': 'btn btn-default btn-sm',
            'data-toggle': 'modal',
            'data-target': '#modalClubInfo',
            'data-club-id': oClub['@name']
        });
        clubEditButton.textContent = "Edit " + oClub['@name'];
        // Add a button to add a athlete
        $(athleteAddButton).attr({
            'id': "edit" + clubButtonID,
            'type': 'button',
            'class': 'btn btn-default btn-sm',
            'data-toggle': 'modal',
            'data-target': '#modalAthleteInfo',
            'data-club-id': oClub['@name']
        });
        athleteAddButton.textContent = "Add athlete";
        // Add a button to add a relay team
        $(relayAddButton).attr({
            'id': "edit" + clubButtonID,
            'type': 'button',
            'class': 'btn btn-default btn-sm',
            'data-toggle': 'modal',
            'data-target': '#modalRelayInfo',
            'data-club-id': oClub['@name']
        });
        /*
        $(relayAddButton).data("eventData", {"text": sessionText,
            "eventId": eventId, "nrAthletes": nrAthletes,
            "gender": oEvent['@gender']});
        */
        relayAddButton.textContent = "TODO Add relay team";
        $(athleteButton).attr({
            'id': athleteButtonID,
            'type': 'button',
            'class': 'btn btn-default',
            'data-toggle': 'collapse',
            'title': 'click to show or hide athletes',
            'data-target': '#' + divAllAthletesID
        });
        athleteButton.textContent = "Athletes";

        $(relayButton).attr({
            'id': relayButtonID,
            'type': 'button',
            'class': 'btn btn-default',
            'data-toggle': 'collapse',
            'title': 'click to show or hide relay teams',
            'data-target': '#' + divAllRelaysID
        });
        relayButton.textContent = "Relays";

        regionText = this.getRegionText(oClub['@region'], oClub['@nation']);
        // TODO Loop through all the items in the club and only add them when
        // they have a value (and are useful)
        divClubInfo.innerHTML = [
                  'Code: ',           oClub['@code'],
            '<br />Name: ',           oClub['@name'],
            '<br />Name (en): ',      oClub['@name.en'],
            '<br />Country: ',        this.getNation(oClub['@nation']),
            '<br />Number: ',         oClub['@number'],
            '<br />Region: ',         regionText,
            '<br />Shortname: ',      oClub['@shortname'],
            '<br />Shortname (en): ', oClub['@shortname.en'],
            '<br />SwrID: ',          oClub['@swrid'],
            '<br />Type: ',           oClub['@type'],
            '<br />'
        ].join('');
        // TODO Add button for "edit/add contact"
        divClubInfo.id = clubInfoID;
        divClubInfo.className = "collapse";
        divClubInfo.appendChild(clubEditButton);
        divClubInfo.appendChild(document.createElement('br'));
        divClubInfo.appendChild(athleteAddButton);
        divClubInfo.appendChild(document.createElement('br'));
        divClubInfo.appendChild(relayAddButton);

        // Replace the items when they are available
        try {
            document.getElementById(clubButtonID).textContent =
                clubButton.textContent;
            document.getElementById(clubInfoID).innerHTML =
                divClubInfo.innerHTML;
        } catch (e) {
            // Add the items when they are not available
            this.clubsInfoNode.insertBefore(clubButton, null);
            this.clubsInfoNode.insertBefore(divClubInfo, null);
        }
        if (oClub.ATHLETES) {
            // only add/show these when there are athletes
            try {
                document.getElementById(athleteButtonID).textContent =
                    athleteButton.textContent;
                document.getElementById(divAthleteInfoID).innerHTML = "";
            } catch (e) {
                // Add the items when they are not available
                this.clubsInfoNode.insertBefore(athleteButton, null);
                divAllAthletes.appendChild(divAthleteInfo);
                this.clubsInfoNode.insertBefore(divAllAthletes, null);
            }
        }
        if (oClub.RELAYS) {
            // only add/show these when there are relays
            try {
                document.getElementById(divRelayInfoID).innerHTML = "";
            } catch (e) {
                this.clubsInfoNode.insertBefore(relayButton, null);
                divAllRelays.appendChild(divRelayInfo);
                this.clubsInfoNode.insertBefore(divAllRelays, null);
            }
        }
        // TODO See how we can see which athlete we are talking about
        this.athleteInfoNode = document.getElementById(divAthleteInfoID);
        this.relayInfoNode = document.getElementById(divRelayInfoID);

        this.currentClubIndex = oClub['@name'];
        if (oClub.ATHLETES) {
            athletes = oClub.ATHLETES.ATHLETE;
            // Loop through the Athletes
            this.handleObjectOrArray(Splash.displaySingleAthlete, athletes,
                this);
        }
        if (oClub.RELAYS) {
            relays = oClub.RELAYS.RELAY;
            // Loop through the relays
            this.handleObjectOrArray(Splash.displaySingleRelay, relays,
                this);
        }
    },

    displayClubs: function() {
        var club = this.getClubData();

        // Loop through the clubs
        this.handleObjectOrArray(this.displaySingleClub, club, this);
        document.getElementById('exportClubs').innerHTML = '\
<button class="btn btn-default btn-sm" onclick="Splash.exportClubs()">Export</button>';
    },

    handleFileSelect: function(evt) {
        var inSender = evt.target.id;
        var files = evt.target.files; // FileList object
        // files is a FileList of File objects. List some properties.
        var output = [];
        var parsedJson, reader, loop, file;
        var isLxfFile = false;

        var aNode = document.getElementById('exportCompetitionLink');
        // Clear the download text, because it is not correct any more
        aNode.textContent = "";
        aNode = document.getElementById('exportClubsLink');
        aNode.textContent = "";

        for (loop = 0; loop < files.length; loop++) {
            file = files[loop];
            reader = new FileReader();

            // Closure to capture the file information.
            reader.onload = (function(theFile, inSender) {
                return function(e) {
                    var dom = {}, json = {}, parsedJson = {};

                    Splash.lastFileSender = inSender;
                    if (isLxfFile) {
                        zip.createReader(new zip.Data64URIReader(e.target.result), zipReaderFunction, onZipError );
                    } else {
                        dom = parseXml(e.target.result);
                        json = xml2json(dom, "");
                        parsedJson = JSON.parse(json);

                        Splash.createDataStructure(parsedJson.LENEX, inSender);
                    }
                };
            })(file, inSender);

            // test if the file is a zip file (.lxf)
            if (file.name.endsWith(".lxf") || file.name.endsWith(".zip")) {
                isLxfFile = true;
                reader.readAsDataURL(file);
            } else {
                reader.readAsText(file);
            }
        }
    },

    // This will call the callback function in a forEach when it is an
    // array and just the callback with argument when it is not an array
    handleObjectOrArray: function(callback, objOrArr, thisArg) {
        if (!callback || !objOrArr || !thisArg) {
            return;
        }
        if (!Array.isArray(objOrArr)) {
            callback.apply(thisArg, [objOrArr, 0, []]);
        } else {
            objOrArr.forEach(callback, thisArg);
        }
    },

    getEventName: function(objSwimstyle) {
        var name = "";

        if (!objSwimstyle) {
            return name;
        }
        name = this.lifesavingStrokes[objSwimstyle['@stroke']];

        switch (name) {
            case "UNKNOWN":
                name = objSwimstyle['@name'];
                break;
            case undefined:
                name = objSwimstyle['@stroke'];
                break;
            default:
                // The name was converted, nothing to do
                break;
        }
        return name;
    },

    getGenderText: function(genderChar) {
        var gender = "";
        switch (genderChar) {
            case GENDER_FEMALE:
                gender = "Female";
                break;
            case GENDER_MALE:
                gender = "Male";
                break;
            case GENDER_MIXED:
                gender = "Mixed";
                break;
            case GENDER_ALL:
            default:
                gender = "All genders";
                break;
        }
        return gender;
    },

    ensureArray: function(data) {
        if (!Array.isArray(data)) {
            // Make it an array
            data = [data];
        }
        return data;
    },

    setWarnBeforeClose: function(value) {
        var aNode = {};

        if (Splash.defaultWarnBeforeClose) {
            Splash.warnBeforeClose = value;
        }
        if (value == true) {
            // Clear the download text, because it is not correct any more
            aNode = document.getElementById('exportCompetitionLink');
            aNode.textContent = "";
            aNode = document.getElementById('exportClubsLink');
            aNode.textContent = "";
        }
    },

    confirmUnload: function(e) {
        var confirmationMessage = "";

        if (Splash.warnBeforeClose) {
            confirmationMessage = "Did you export the changes?";
        }

        e.returnValue = confirmationMessage;
        return confirmationMessage;
    }
    }; // end of 'var Splash ='

    Splash.init();

    // We only need these events when splash.html is viewed
    if (Splash.splashHtml) {
        document.getElementById('meetFile').addEventListener('change',
            Splash.handleFileSelect, false);
        document.getElementById('clubFile').addEventListener('change',
            Splash.handleFileSelect, false);
        window.addEventListener("beforeunload", Splash.confirmUnload, false);
    }
})();
// vim: set ai ts=4 sw=4 et:
